;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : Compressor gain without smoothing parameter (only hard knee)
;
; Description : Calculates the gain to apply to compress the incoming signal
;
; Nb In       : 1
; Nb Out      : 1
;
; Input       : audio signal in ABSOLUTE VALUE!!!
; Output      : compressor gain
;
; Cycles      : 14 + 15 cycles (-1=unknownw)
; Memory      : 5 words (without init)
; Delay Line  : no
;
; Author      : Gregor
; Date        : 13/09/2003
; Version     : 1.0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ABS     INIT    0x3FF

        ;; SEC1 Control parameters

ABS TShold  0x0		; Compression threshold
ABS Ratio   0x1		; Compression factor
ABS Tac     0x2		; Attack time
ABS Trc	    0x3		; Release time

        ;; SEC2 Initial values of control parameters (S3.24 format)

ITShold	EQU -0.75			; {0,1,log}
IRatio	EQU 0.75			; {0,1,log}
ITac	EQU 1.0				; {0,1,log}
ITrc	EQU 1.0				; {0,1,log}


        ;; SEC3 Constants definitions
; none

        ;; SEC4 Memory variables definition
ABS Gc	    0x4		; Current gain

        ;; SEC5 Initial values of memory variables
IGc		EQU 1.0				; {0,1,log}

        ;; SEC6 Initialization
CM      0x40000 INIT
SKIP    !Z      EndInit
CM      0.0     INIT
1AC     ITShold			    ; 
SCA     0x0     TShold		; init Tshold
1AC     IRatio			    ; 
SCA     0x0     Ratio		; init Ratio
1AC     ITac		        ; 
SCA     0x0     Tac			; init Tac
1AC     ITrc		        ; 
SCA     0x0     Trc			; init Trc
1AC     IGc		            ; 
SCA     0x0     Gc			; init Gc
DAC     0x0     0x40000     ; 1->A
SCA     0x0     INIT        ; do not reinitialize next time
EndInit:
	
        ;; SEC7 Module code
		
LCM	    0x0     IN1
LOGB
CAM	    -1.0    TShold
SKIP	!N	    Low

High:
AMC	    0x0     Ratio
XCM	    1.0     Tac         ; temp -> B, Tattack -> A
SKIP    End
	
Low:
DAC	    0x0     0x0			; out=in
XCM	    1.0     Trc         ; temp -> B, Trelease -> A

End:	
SEXPB	Dir0			    ; exp(Temp)->A, t*c->Dir0

	;; first order filter
LCMA	-1.0    Gc
AMB	    Dir0			    ; new Gc value in A
SCA	    1.0		 Gc		    ; ... and in Gc
SCA     1.0		OUT1	    ; store