;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : Simple Envelope follower
;
; Description : Getting the envelope of a signal based on one decay time
;                 
; Nb In       : 1
; Nb Out      : 1
;
; Input       : audio signal
; Output      : envelope of the signal
;
; Cycles      : 7 + 8 cycles (-1 = unknown)
; Memory      : 2 words (without init)
; Delay Line  :	no
;
; Author      : Lall
; Date        : 13/09/2003
; Version     : 1.0
;
; Control parameters (see SEC1): Decay
;	gt = (float) exp(-1 / (SampleRate * attack_in_sec))
;	decay = 1 - gt
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;; SEC0 General Parameters
ABS	INIT		0x3ff		; init params at first run

	;; SEC1 Control Parameters
ABS	    Decay   0x000

	;; SEC2 Initial values of control parameters (S3.24 format)
IDecay	EQU	    0xfd5732		;  float gt = (float) exp(-1/(SampleRate*attack)); 
								;  decay = 1 - gt

	;; SEC3 Constants definitions
;none

	;; SEC4 Memory variables definition
ABS	    LastEnv	0x001

	;; SEC5 Initial values of memory variables
ILastEnv    EQU	0.0

	;; SEC6 Initialization : (should restore register to 0x0)
CM      0x40000 INIT
SKIP	!Z		EndInit
C		ILastEnv			; load ILastEnv
SCA 	0.0  	LastEnv		; save to LastEnv
C		IDecay				; load IDecay
SCA 	0.0  	Decay		; save to Decay
C     0x1000000     ; 1->A
SCA     0x0     INIT        ; do not reinitialize next time
EndInit:

	;; SEC7 Main
;EnvIn = abs(input); 
CM		1.0		IN1			; A contains the input
SKIP	!N		1			; if the number is positive don't do next line
LCA		-1.0	LastEnv		; invert A if number if negative
							; A = AbsIn
							; B = LastEnv

;LastEnv = LastEnv + (AbsIn - LastEnv) * Decay
CMA		-1.0	LastEnv		; A = AbsIn - LastEnv
AMB				Decay		; A = LastEnv + (AbsIn - LastEnv) * Decay

SCA		1.0		LastEnv		; store the last envelope 
SCA		0.0		OUT1		; store the output
