;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : Linear interpolation
;
; Description : Interpolates in a linear fashion a value between two extremes
;                 based on a given ratio between delay min and delay max
; Nb In       : 1
; Nb Out      : 1
;
; Input       : 1 channel signal, interpolation ratio between dMin and dMax
; Output      : 1 channel signal, the interpolated value
;
; Cycles      : 6 + 8 cycles (-1=unknownw)
; Memory      : 2 words (without init)
; Delay Line  :	no
;
; Author      : Lall
; Date        : 23/03/2003
; Version     : 1.0
;
; Control parameters (see SEC1): vMin and vMax
;	dMin: minumum "delay" for the phaser, given as normalized frequency i.e. dMin = fMin / (Fs/2)
;	dMax: maximum "delay" for the phaser, given as normalized frequency i.e. dMax = fMax / (Fs/2)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;; SEC0 General Parameters
ABS		INIT		0x3ff 	; init params at first run

	;; SEC1 Control Parameters
ABS		dMin		0		; Minimum delay
ABS		dMax		1		; Maximum delay


	;; SEC2 Initial values of control parameters (S3.24 format)
IdMin		EQU 	0.02	; 480 Hz  = min frequency
IdMax		EQU		0.1		; 2400 Hz = max frequency

	;; SEC3 Constants definitions

	;; SEC4 Memory variables definition

	;; SEC5 Initial values of memory variables

	;; SEC6 Initialization : (should restore register to 0x0)
CM	0x40000 INIT
SKIP	!Z	EndInit
C	IdMin			; load init value for IdMin ...
SCA	0x0 dMin		; ... Stored in dMin
C	IdMax			; load init value for IdMax ...
SCA	0x0 dMax 		; ... Stored in dMax
C       0x1000000     ; 1->A
SCA     0x0     INIT        ; do not reinitialize next time
EndInit:

	;; SEC7 Main

;;; Interpolate the value
CM		0.5		dMax		; A = dMax / 2.0
CMA		-0.5	dMin		; A = (dMax - dMin) / 2.0
AMC		0.0		IN1			; A = IN1 * (dMax - dMin) / 2.0
CMA		0.5		dMin		; A = dMin/2 + IN1 * (dMax - dMin) / 2.0
CMA		0.5		dMax		; A = dMax/2 - dMin/2 + IN1 * (dMax - dMin) / 2.0
SCA		1.0		OUT1		; OUT1 = dMin + IN1 * (dMax - dMin) / 2.0   with IN1 in [-1.0 ... 1.0]
							; OUT1 = dMin + (dMax - dMin) * ratio   with ratio in [0.0 ... 1.0]

;; dMin + (dMax - dMin) * ratio   where ratio is in [0.0 ... 1.0]
;; = dMin + (dMax - dMin) * (IN1 + 1.0) / 2.0   where IN1 is in [-1.0 ... 1.0]
;; = dMin + (dMax - dMin) / 2.0 + IN1 * (dMax - dMin) / 2.0
;; = dMin/2 + dMax/2 + IN1 * (dMax - dMin)/2