;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : All-pass calculate coefficient
;
; Description : Calculates the coefficients for an all-pass filter based on a given delay
;                 
; Nb In       : 1
; Nb Out      : 1
;
; Input       : 1 channel signal, current delay (output of interpolator)
; Output      : 1 channel signal, coefficient for the all-pass
;
; Cycles      : 12 + 0 cycles (-1=unknownw)
; Memory      : 1 words (without init)
; Delay Line  :	no
;
; Author      : Lall
; Date        : 23/03/2003
; Version     : 1.0
;
; Control parameters (see SEC1): none
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;; SEC0 General Parameters
ABS	INIT	0x3ff		; init params at first run

	;; SEC1 Control Parameters

	;; SEC2 Initial values of control parameters (S3.24 format)

	;; SEC3 Constants definitions

	;; SEC4 Memory variables definition
ABS		TempValue	0x000	; Uninitialized temporary memory

	;; SEC5 Initial values of memory variables

	;; SEC6 Initialization : (should restore register to 0x0)

	;; SEC7 Main

;;; Generating then the all-pass coefficients from that
1MC		1.0		IN1			; A = 1 + x
XCM		0.0		TempValue	; A -> B		A = 0 * TempValue
LOGB						; A = log16(1 + x)
LCA		-1.0	TempValue	; A = -log16(1 + x)	B = dummy
XCM		0.0		TempValue	; A -> B		A = 0 * TempValue
EXPB						; A = ~ 1 / (1 + x)
SCA		0.0		TempValue	; mem[TempValue] = ~ 1 / (1 + x)
LCA		0.0		IN1			; B = x (= mem[coeff])
CB		-1.0				; A = -x
1AC		1.0					; A = 1 - x
AMC		0.0		TempValue	; A = (1 - x) * (~ 1 / (1 + x))
SCA		0.0		OUT1		; mem[coeff] = A

