;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : random15
;
; Description : 15-bits random number generator generator
;
; Nb In       : 0
; Nb Out      : 1
;
; Input       : None
; Output      : 16-bits random number between 0 and 1
;
; Cycles      : 6 + 10 cycles (-1=unknownw)
; Memory      : 1 words (without init)
; Delay Line  : no
;
; Author      : Gregor
; Date        : 05/02/2005
; Version     : 2.0
;
; This is implemented using the usual recursive relation
;    x_n = (a x_n-1 + c) mod 2^16
; Note that the chosen values of a and c ensure a maximal periodicity
; and passes rather successfully a 2-dimensional lattice test
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ABS	INIT	0x3FF
	
	;; SEC1 Control parameters
ABS	Value	0x0
	
	;; SEC2 Initial values of control parameters (S3.24 format)
IValue	EQU	0x0f12200	; 30865
	
	;; SEC3 Constants definitions
CMult	EQU	0x6c21000	; 27681 in integer form for MULTB
CAdd	EQU	0x193a8		; 12917 in S3.18 (!!!!) format (for add)
CMask0	EQU	0x7fff000	; keep only relevant bits
CMask	EQU	0x0ffff00	; keep only relevant bits
	
	;; SEC4 Memory variables definition

	;; SEC5 Initial values of memory variables

	;; SEC6 Initialization
CM	0x40000	INIT
SKIP	!Z	EndInit
C	IValue
SCA	0x0 Value
C       0x1000000		; 1->A
SCA     0x0     INIT		; do not reinitialize next time
EndInit:

	;; SEC7 Module code
C	CMult			; load 'a'
XCM	0x40000 Value		; store it in B and get previous value
CAD	0x100000 0x0		; shift 2 bits to the left
CAD	0x80000 0x0		; shift 1 bits to the left
MLTB				; a x_n
ANDC	CMask0			; keep relevant bits
DAC	0x20 CAdd 		; back-shift and add 'c'
ANDC	CMask			; keep relevant bits
SCA	0x40000 Value		; save for next run
SCA	0x0 OUT1
