#include "AL3101ModuleHeader.h"
#include <stdio.h>
#include <string.h>
//#include "console.h"

// gets the number of inputs and outputs for the module "name"
int GetModuleInfo(char *name, TModuleInfo *info){
#ifdef DEBUG
  fprintf(stderr, "DBG: AL3101ModuleHeader: Performing GetModuleInfo.\n");
#endif

  FILE *flux=fopen(name, "r");
  char Line[255], *Plus;
  int RetCode, len;

  if (flux==NULL){
    //wxLogMessage("Cannot open module file");
    goto Error_total;
  }

  // Scan for module name
  RetCode=0;
  while (RetCode<=0){
    if ((fgets(Line,255,flux)==NULL) || (Line[0]!=';')){
      fclose(flux);
      goto Error_Name;
    }
    if (strncmp(Line, "; Effect Name :", 15)==0){
      RetCode = 1;
      strncpy(info->Name, Line+16, len=(strlen(Line)-16));
      info->Name[len] = '\0';
      while ((info->Name[len-1]==0x0A) || (info->Name[len-1]==0x0D)){ len--;info->Name[len]='\0';};
    }
  }

  // Scan for module description
  RetCode=0;
  while (RetCode<=0){
    if ((fgets(Line,255,flux)==NULL) || (Line[0]!=';')){
      fclose(flux);
      goto Error_Descr;
    }
    if (strncmp(Line, "; Description :", 15)==0){
      RetCode = 1;
      strncpy(info->Descr, Line+16, len=(strlen(Line)-16));
      info->Descr[len] = '\0';
      while ((info->Descr[len-1]==0x0A) || (info->Descr[len-1]==0x0D)){ len--;info->Descr[len]='\0';};
    }
  }

  // scan for Nb In ?
  RetCode=0;
  while (RetCode<=0){
    if ((fgets(Line,255,flux)==NULL) || (Line[0]!=';')){
      fclose(flux);
      goto Error_NbIn;
    }
    RetCode=sscanf(Line, "; Nb In       : %d", &(info->NbIn));
  }

  // scan for Nb Out ?
  RetCode=0;
  while (RetCode<=0){
    if ((fgets(Line,255,flux)==NULL) || (Line[0]!=';')){
      fclose(flux);
      goto Error_NbOut;
    }
    RetCode=sscanf(Line, "; Nb Out      : %d", &(info->NbOut));
  }

  // Scan for input description
  RetCode=0;
  while (RetCode<=0){
    if ((fgets(Line,255,flux)==NULL) || (Line[0]!=';')){
      fclose(flux);
      goto Error_InDescr;
    }
    if (strncmp(Line, "; Input       :", 15)==0){
      RetCode = 1;
      strncpy(info->InDescr, Line+16, len=(strlen(Line)-16));
      info->InDescr[len] = '\0';
      while ((info->InDescr[len-1]==0x0A) || (info->InDescr[len-1]==0x0D)){ len--;info->InDescr[len]='\0';};
    }
  } 
  
  // Scan for output description
  RetCode=0;
  while (RetCode<=0){
    if ((fgets(Line,255,flux)==NULL) || (Line[0]!=';')){
      fclose(flux);
      goto Error_OutDescr;
    }
    if (strncmp(Line, "; Output      :", 15)==0){
      RetCode = 1;
      strncpy(info->OutDescr, Line+16, len=(strlen(Line)-16));
      info->OutDescr[len] = '\0';
      while ((info->OutDescr[len-1]==0x0A) || (info->OutDescr[len-1]==0x0D)){ len--;info->OutDescr[len]='\0';};
    }
  } 

  // scan for cycles
  // NOTE: can be given under two different forms
  //   1. <n> cycles
  //   2. <n> + <i> cycles
  // in the second one, core and init cycles are given separately.
  RetCode=0;
  while (RetCode<=0){
    if ((fgets(Line,255,flux)==NULL) || (Line[0]!=';')){
      fclose(flux);
      goto Error_Cycles;
    }
    RetCode=sscanf(Line, "; Cycles      : %d", &(info->Cycles));
    Plus = strchr(Line, '+');
    len = 0;
    if ((RetCode == 1) && (Plus!=NULL))
      len=sscanf(Plus+1, "%d", &(info->InitCycles));
    if ((len==0) || (info->InitCycles<4)) info->InitCycles = 4;
  }

  // scan for memory usage
  RetCode=0;
  while (RetCode<=0){
    if ((fgets(Line,255,flux)==NULL) || (Line[0]!=';')){
      fclose(flux);
      goto Error_Mem;
    }
    RetCode=sscanf(Line, "; Memory      : %d", &(info->MemUsage));
  }

  // Scan for delay line needs
  RetCode=0;
  while (RetCode<=0){
    if ((fgets(Line,255,flux)==NULL) || (Line[0]!=';')){
      fclose(flux);
      goto Error_Delay;
    }
    if (strncmp(Line, "; Delay Line  :", 15)==0){
      RetCode = 1;
      if (strncmp(Line+16, "yes", 3)==0)	
	info->UseDelay = 1;
      else
	info->UseDelay = 0;
    }
  }

  //fprintf(stdout,"Module: %s\nDescr  : %s\nCycles : %d + %d\nMem : %d\n\n", info->Name, info->Descr, info->Cycles, info->InitCycles, info->MemUsage);

  fclose(flux);
  return MODULE_HEADER_OK;

 Error_total:
 Error_Name:
  sprintf(info->Name, "?");
 Error_Descr:
  sprintf(info->Descr, "?");
 Error_NbIn:
  info->NbIn=1;
 Error_NbOut:
  info->NbOut=1;
 Error_InDescr:
  sprintf(info->InDescr, "?");
 Error_OutDescr:
  sprintf(info->OutDescr, "?");
 Error_Cycles:
  info->Cycles=-1;
  info->InitCycles=4;
 Error_Mem:
  info->MemUsage=0;
 Error_Delay:
  info->UseDelay=0;

  return MODULE_HEADER_ERROR;
}
