/////////////////////////////////////////////////////////////////////////////
// Name:        AL3101DealProperties.cpp
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.0
// Created:     24/11/02
// Copyright:   (c) 2002 Axoris
/////////////////////////////////////////////////////////////////////////////

#include "AL3101DealProperties.h"
#include <string.h>
#include <math.h>
#include <ctype.h>

// scan file to search for section 2
// Then fill the properties array for that module
// We return the number of properties.
int GetPropertyArray(FILE *flux, TModuleProperty **mp){
  char Line[255], PropName[255], PropId[255], PropValue[255], PropDescr[255];
  int stop, nb, i,res;
  long StartPoint=0;

  stop=0;
  while (!stop){
    if (fgets(Line,255,flux)==NULL) return 0;
    if (strstr(Line, ";; SEC1")!=NULL){
      StartPoint=ftell(flux);
      stop=1;
    }
  };

  // scan for the number of properties
  nb=0;
  stop=0;
  while(!stop){
    // read line from file, split it and check that it has the form
    // ABS <name> <value>
    if (fgets(Line,255,flux)==NULL) return 0;
    i=sscanf(Line, "%s %s %s", PropId, PropName, PropValue);
    if (strstr(Line, ";; SEC")!=NULL){
      stop=1;
    } else {
      if ((i==3) && 
	  ((strcmp(PropId, "ABS")==0) || 
	   (strcmp(PropId, "Abs")==0) || 
	   (strcmp(PropId, "abs")==0))){
	// correct property
	nb++;
      }
    }
  };

  // if no property found, directly return
  if (nb==0) return 0;

  // allocate memory for properties;
  (*mp)=new TModuleProperty[nb];
  
  // read properties
  fseek(flux, StartPoint, SEEK_SET);
  // fill properties array
  nb=0;
  stop=0;
  while(!stop){
    // read line from file, split it and check that it has the form
    // ABS <name> <value>
    if (fgets(Line,255,flux)==NULL) return 0;
    i=sscanf(Line, "%s %s %s", PropId, PropName, PropValue);
    if (strstr(Line, ";; SEC")!=NULL){
      stop=1;
    } else {
      if ((i==3) && 
	  ((strcmp(PropId, "ABS")==0) || 
	   (strcmp(PropId, "Abs")==0) || 
	   (strcmp(PropId, "abs")==0))){
	// correct property
	strcpy((*mp)[nb].Name, PropName);
	strcpy((*mp)[nb].Address, PropValue);
	// extract the value of the address
	// we assume it is an S3.24 format
	// convert string to upper case
	for (res=0;res<strlen(PropValue);res++) 
	  PropValue[res] = (char) toupper(PropValue[res]);
	res = ExtractHexa(PropValue, &((*mp)[nb].address), -1);
	// check if we have an additionnal description of the param
	i=strlen(Line);
	while ((Line[i]=='\n') || (Line[i]=='\r')){ Line[i]='\0'; i--;}
	i=0;
	while ((i<strlen(Line)) && (Line[i]!=';')) i++;
	if (i<strlen(Line)-1){
	  strncpy(PropDescr, Line+i+2, strlen(Line)-1);
	  i = strlen(PropDescr);
	  while ((PropDescr[i-1]=='\n') || (PropDescr[i-1]=='\r')){
	    PropDescr[i-1] = '\0';
	    i--;
	  };
	} else
	  sprintf(PropDescr, "");
	strcpy((*mp)[nb].Descr, PropDescr);
	strcpy((*mp)[nb].Value, "0x0");
	(*mp)[nb].value = 0;
	nb++;
      }
    }
  };

  // search for the initial values of the properties
  fseek(flux, StartPoint, SEEK_SET);
  stop=0;
  while (!stop){
    if (fgets(Line,255,flux)==NULL) return 0;
    if (strstr(Line, ";; SEC2")!=NULL){
      StartPoint=ftell(flux);
      stop=1;
    }
  };

  // read property values
  fseek(flux, StartPoint, SEEK_SET);
  // fill properties array
  stop=0;
  while(!stop){
    // read line from file, split it and check that it has the form
    // <name> EQU <value>
    if (fgets(Line,255,flux)==NULL) return 0;
    i=sscanf(Line, "%s %s %s", PropName, PropId, PropValue);
    if (strstr(Line, ";; SEC")!=NULL){
      stop=1;
    } else {
      if ((i==3) && 
	  ((strcmp(PropId, "EQU")==0) || 
	   (strcmp(PropId, "Equ")==0) || 
	   (strcmp(PropId, "equ")==0))){
	// correct property
	// find if it corresponds to an existing address
	i=0;
	while ((i<nb) && (strcmp((*mp)[i].Name, PropName+1)!=0)) i++;
	if (i<nb){
	  // match found
	  strcpy((*mp)[i].Value, PropValue);
	  // extract the value of the constant
	  // if the constant is not given in floating point format
	  // we assume it is an S3.24 format
	  // convert string to upper case
	  for (res=0;res<strlen(PropValue);res++) 
	    PropValue[res] = (char) toupper(PropValue[res]);
	  res = ExtractHexa(PropValue, &((*mp)[i].value), 24);
	}
      }
    }
  };

  return nb;
}


float ConvertToFloat(UINT32 s){
  UINT32 neg;
  float res;

  neg = (s>>27);
  if (neg>0){
    s=((neg+1) << 27)-s;
  }
  res = s*pow(0.5,24);
  if (neg>0)
    res*=-1.0;

  return res;
}
