/////////////////////////////////////////////////////////////////////////////
// Name:        frame.cpp
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     04/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#include "gldefs.h"
#include "frame.h"
#include "about.h"
#include "print.h"
//LOAD#include "load.h"
#include "directories.h"
#include "simulate.h"
#include "mpproject.h"
#include "ProjectInfo.h"
#include "theme.h"

#ifndef WIN32
#include <config.h>
#endif

//#define DEFAULT_PATH "../effects"
// ----------------------------------------------------------------------------
// resources
// ----------------------------------------------------------------------------
// the application icon
#if defined(__WXGTK__) || defined(__WXMOTIF__)
    #include "resources/AgALag/mondrian.xpm"
#endif

//#include <wx/string.h>

// ----------------------------------------------------------------------------
// event tables and other macros for wxWindows
// ----------------------------------------------------------------------------

// the event tables connect the wxWindows events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// simple menu events like this the static method is much simpler.
BEGIN_EVENT_TABLE(MainFrame, wxFrame)
  EVT_MENU(Menu_New,       MainFrame::OnNew)
  EVT_MENU(Menu_Open,      MainFrame::OnOpen)
  EVT_MENU(Menu_Save,      MainFrame::OnSave)
  EVT_MENU(Menu_SaveAs,    MainFrame::OnSaveAs)
  EVT_MENU(Menu_Print,     MainFrame::OnPrint)
  EVT_MENU(Menu_Quit,      MainFrame::OnQuit)
  EVT_MENU(Menu_ClearC,    MainFrame::OnClearConsole)
  EVT_MENU(Menu_Make,      MainFrame::OnMake)
  EVT_MENU(Menu_Assemble,  MainFrame::OnAssemble)
  EVT_MENU(Menu_MAsm,      MainFrame::OnMAsm)
  EVT_MENU(Menu_Info,      MainFrame::OnProjectInfo)
  EVT_MENU(Menu_Options,   MainFrame::OnProjectOptions)
  EVT_MENU(Menu_Simulate,  MainFrame::OnSimulate)
  EVT_MENU(Menu_MissParker,MainFrame::OnMissParker)
  //LOADEVT_MENU(Menu_Load,      MainFrame::OnLoad)
  EVT_MENU(Menu_CfgDirs,   MainFrame::OnCfgDirs)
  EVT_MENU(Menu_CfgColors, MainFrame::OnCfgColors)
  EVT_MENU(Menu_Help,      MainFrame::OnHelp)
  EVT_MENU(Menu_BugReport, MainFrame::OnBugReport)
  EVT_MENU(Menu_About,     MainFrame::OnAbout)
  EVT_TREE_BEGIN_DRAG(IDWinModList, MainFrame::OnTreeDrag)
END_EVENT_TABLE()


// ----------------------------------------------------------------------------
// implementation for the main frame
// ----------------------------------------------------------------------------

// frame constructor
MainFrame::MainFrame()
  : wxFrame((wxFrame *)NULL, -1, _T("AgALag development interface"), 
            wxDefaultPosition){
#ifdef DEBUG
  fprintf(stderr, "DBG: frame: Entering constructor\n");
#endif

#ifdef __WXMAC__
  // we need this in order to allow the about menu relocation, since ABOUT is
  // not the default id of the about menu
  wxApp::s_macAboutMenuItemId = Menu_About;
#endif
  
  // set the frame icon
  SetIcon(wxICON(mondrian));

  CreateMenu();
  CreateStatus();
  if (Split()!=0)
    IsInitialised=0;
  else {
    Maximize(TRUE);
    IsInitialised=1;
  }
}

MainFrame::~MainFrame(){
#ifdef DEBUG
  fprintf(stderr, "DBG: frame: Entering destructor\n");
#endif
  CConfig *Config;
  Config=new CConfig;

  int width, height;
  char *tmp1, *tmp2;

  GetSize(&width, &height);
  Config->WindowWidth=width;
  Config->WindowHeight=height;

  // frames size
  Config->StructWidth = Upper->GetSashPosition();
  Config->StructHeight = Total->GetSashPosition();

  // project options
  Config->AsmVerbose = ProjectOptions.AsmVerbose;
  Config->MkVerbose = ProjectOptions.MkProjVerboseLevel;
  Config->MkMemMap = ProjectOptions.MkProjMem;

  tmp1 = Config->ProjectName;
  tmp2 = new char[strlen(Project->Name)+1];
  strcpy(tmp2, Project->Name);
  Config->ProjectName=tmp2;
  delete tmp1;

  delete Config;

  delete Project;
}

/*
** Creates the main frame menu and its items
*/
int MainFrame::CreateMenu(){
#ifdef DEBUG
  fprintf(stderr, "DBG: frame: Entering CreateMenu\n");
#endif

  // create a menu bar
  // File Menu
  MenuFile = new wxMenu();
  MenuFile->Append(Menu_New,    _T("&New\tCtrl-N"),     _T("New project"));
  MenuFile->Append(Menu_Open,   _T("&Open\tCtrl-O"),    _T("Open existing project"));
  MenuFile->Append(Menu_Save,   _T("&Save\tCtrl-S"),    _T("Save current project"));
  MenuFile->Append(Menu_SaveAs, _T("Save &As\tCtrl-A"), _T("Save project with a new name"));
  MenuFile->AppendSeparator();
  MenuFile->Append(Menu_Print , _T("&Print\tCtrl-P"),   _T("Print current project"));
  MenuFile->AppendSeparator();
  MenuFile->Append(Menu_Quit,   _T("E&xit\tAlt-X"),     _T("Quit this program"));

  // Edit Menu
  MenuEdit = new wxMenu();
  MenuEdit->Append(Menu_ClearC, _T("&Clear Console\tCtrl-X"),   _T("Clears the console"));

  // Project Menu
  MenuProject = new wxMenu();
  MenuProject->Append(Menu_Make,     _T("&Make\tAlt-M"),        _T("Make the project"));
  MenuProject->Append(Menu_Assemble, _T("A&ssemble\tAlt-A"),      _T("Assemble the source"));
  MenuProject->Append(Menu_MAsm    , _T("Make+&Assemble\tCtrl-Shift-A"), _T("Make and assemble the source"));
  MenuProject->AppendSeparator();
  MenuProject->Append(Menu_Info    , _T("Settings\tAlt-S"), _T("Project Informations"));
  MenuProject->Append(Menu_Options , _T("Options\tAlt-O"), _T("Project Options"));
  MenuProject->AppendSeparator();
  MenuProject->Append(Menu_Simulate, _T("Simulate\tAlt-I"), _T("Launch with alksim"));
  MenuProject->Append(Menu_MissParker, _T("Miss Parker\tAlt-"), _T("Miss Parker project"));
  //LOADMenuProject->Append(Menu_Load    , _T("Load\tCtrl-L"), _T("load the code"));

  // Options Menu
  MenuOptions = new wxMenu();
  MenuOptions->Append(Menu_CfgDirs,   _T("&Directories"), _T("Sets directories structure"));
  MenuOptions->Append(Menu_CfgColors, _T("&Theme"),       _T("Choose rendering theme"));

  // the "About" item should be in the help menu
  MenuHelp = new wxMenu();
  MenuHelp->Append(Menu_Help,      _T("&Help\tCtrl-H"), _T("Show help"));
  MenuHelp->Append(Menu_BugReport, _T("&Report Bug"),   _T("Fill a bug report"));
  MenuHelp->AppendSeparator();
  MenuHelp->Append(Menu_About,     _T("&About"),        _T("About AGAD"));
  
  // now append the freshly created menu to the menu bar...
  MenuBar = new wxMenuBar();
  MenuBar->Append(MenuFile,    _T("&File"));
  MenuBar->Append(MenuEdit,    _T("&Edit")); 
  MenuBar->Append(MenuProject, _T("&Project"));
  MenuBar->Append(MenuOptions, _T("&Options"));
  MenuBar->Append(MenuHelp,    _T("&Help"));
  
  // ... and attach this menu bar to the frame
  SetMenuBar(MenuBar);
  
  return 0;
}

int MainFrame::CreateStatus(){
#ifdef DEBUG
  fprintf(stderr, "DBG: frame: Entering CreateStatus\n");
#endif

#if wxUSE_STATUSBAR
  // create a status bar (by default with 1 pane only)
  CreateStatusBar(4);
  SetStatusText("Welcome to AgALag!");
#endif // wxUSE_STATUSBAR
  
  return 0;
}

/*
** Splitting the window in 
**  - modules tree,
**  - GL canvas for graphical structure construction
**  - console
*/
int MainFrame::Split(){
#ifdef DEBUG
  fprintf(stderr, "DBG: frame: Entering Split\n");
#endif

  wxSize WinSize;
  int RightSize, TotalWidth;
  CConfig *Config;
  int StatusWidths[4];
  wxRect pos;

  Config = new CConfig;
  // check config parameters
  if (Config->WindowHeight<Config->StructHeight)
    Config->StructHeight = Config->WindowHeight*2/3;
  if (Config->WindowWidth<Config->StructWidth)
    Config->StructWidth = Config->WindowWidth/3;

  // load the options
  ProjectOptions.AsmVerbose = Config->AsmVerbose;
  ProjectOptions.MkProjVerboseLevel = Config->MkVerbose;
  ProjectOptions.MkProjMem = Config->MkMemMap;
  ProjectOptions.MkProjLab = 0;

  // Get Client size of the frame
  SetSize(Config->WindowWidth, Config->WindowHeight);
  WinSize=GetClientSize();

  // resize status bar and create usage bars
  StatusWidths[3]=-1;
  StatusWidths[0]=150;
  StatusWidths[1]=100;
  StatusWidths[2]=100;
  SetStatusWidths(4,StatusWidths);
  GetStatusBar()->GetFieldRect(1,pos);
  DSPStatus = new wxGauge(GetStatusBar(),-1,1024,
			  pos.GetPosition(),pos.GetSize(), 
			  wxGA_HORIZONTAL);
  GetStatusBar()->GetFieldRect(2,pos);
  MemStatus = new wxGauge(GetStatusBar(),-1,1024,
			  pos.GetPosition(),pos.GetSize(), 
			  wxGA_HORIZONTAL);

  // create main splitter
  Total = new wxSplitterWindow(this, IDWinTotal, 
			       wxDefaultPosition, 
			       WinSize);

  // create its childs
  Upper = new wxSplitterWindow(Total, IDWinUpper, 
			       wxDefaultPosition, 
			       wxDefaultSize);
  Console = new wxTextCtrl(Total, IDWinConsole, "", 
			   wxDefaultPosition, 
			   wxDefaultSize, 
			   wxTE_MULTILINE | wxTE_READONLY);

  AppConsole=new wxLogTextCtrl(Console);
  AppConsole->SetActiveTarget(AppConsole);
  AppConsole->SetTimestamp(NULL);

  // split horizontally
  Total->SplitHorizontally(Upper, Console, Config->StructHeight);

  // Get theme
  if (LoadTheme(Config->SkinName, Config->SkinDir, &theme)!=0){
    // no theme loaded ... ask user to select one ...
    if (GetThemeFromUser(Config, &theme)!=0){
      delete Config;
      return 1;
    }
  }

  // ceates the tree and the device context for structure
  WinSize=Upper->GetSize();
  Struct = new CglStruct(Upper, &theme, DSPStatus, MemStatus, IDWinGLStruct, wxDefaultPosition, wxDefaultSize);
  //wxVSCROLL | wxHSCROLL);
  ModuleList = new CModuleList(Upper, IDWinModList, wxDefaultPosition, 
  			       wxDefaultSize, Config->EffectsDir, Struct);

  // Initializing project, theme and struct
  Project = new CFile(Struct->Modules, Struct->LLink, strlen(Config->EffectsDir), this);
  Struct->InitStruct(Project);

  // make second split ... vertically this time.
  TotalWidth=WinSize.GetWidth();
  RightSize=TotalWidth-Config->StructWidth;
  Upper->SplitVertically(ModuleList, Struct, Config->StructWidth);

  Total->SetMinimumPaneSize(5);
  Upper->SetMinimumPaneSize(5);

  // showing struct
  Struct->Show(TRUE);

  delete Config;

  return 0;
}


// event handlers

// File Menu
void MainFrame::OnNew(wxCommandEvent& event){
  Project->New();
  Struct->Redraw();
}

void MainFrame::OnOpen(wxCommandEvent& event){
  Project->Open();
  Struct->Redraw();
}

void MainFrame::OnSave(wxCommandEvent& event){
  Project->Save();  
}

void MainFrame::OnSaveAs(wxCommandEvent& event){
  Project->SaveAs();
}

void MainFrame::OnPrint(wxCommandEvent& event){
  /*wxPrintPreview *preview = new wxPrintPreview(new CPrintStruct(Struct->Modules, Project->info.Name), new CPrintStruct(Struct->Modules, Project->info.Name));
  wxPreviewFrame *frame = new wxPreviewFrame(preview, (wxFrame*) this);
  frame->Centre(wxBOTH);
  frame->Initialize();
  frame->Show(TRUE);
  */

  static wxPrintData PrintData;
  wxPrintDialogData printDialogData(PrintData);

  wxPrinter printer(& printDialogData);
  CPrintStruct printout(Struct->Modules, Project->info.Name);
  if (printer.Print((wxFrame*) this, &printout, TRUE)){
    PrintData = printer.GetPrintDialogData().GetPrintData();
    //fprintf(stdout, "Printing done Sucessfully\n");
  }
}

void MainFrame::OnQuit(wxCommandEvent& WXUNUSED(event)){
#ifdef DEBUG
  fprintf(stderr, "DBG: frame: Entering quit\n");
#endif
  wxLogMessage("Exiting ...");
  // TRUE is to force the frame to close
  Close(TRUE);

}

// Edit Menu
void MainFrame::OnClearConsole(wxCommandEvent& event){
  Console->Clear();
}

// Project Menu
void MainFrame::OnMake(wxCommandEvent& WXUNUSED(event)){
#ifdef DEBUG
  fprintf(stderr, "DBG: frame: Entering OnMake\n");
#endif
  // we must execute the command
  //   ./mkalproj <projectname>

  int ToSave;
  if ((Project->IsModified) || (Struct->Modules->IsModified))
    ToSave = wxMessageBox("Save actual project ?", "Save?", wxYES_NO);
  else
    ToSave=wxNO;
  if (ToSave == wxYES)
    Project->Save();

  wxArrayString Output, Error;
  wxString Command;
  int n,i;

#ifdef WIN32
  Command << "./mkalproj -t ";
#else
  Command << BIN_INSTALL_DIR << "/mkalproj -t ";
#endif

  if (ProjectOptions.MkProjVerboseLevel==1) Command << "-v ";
  if (ProjectOptions.MkProjVerboseLevel==2) Command << "-V ";
  if (ProjectOptions.MkProjLab) Command << "-l ";
  if (ProjectOptions.MkProjMem) Command << "-m ";
  Command << "\"";
  Command << Project->Name;
  Command << "\"";


#ifdef WIN32
  wxExecute(Command, Output, Error);
  n=Output.GetCount();
  for (i=0;i<n;i++)
    (*Console) << Output[i] << "\n";
  n=Error.GetCount();
  for (i=0;i<n;i++)
    (*Console) << Error[i] << "\n";
#else
  (*Console) << "Executing '" << Command << "'\n";
  wxShell(Command);
#endif

  (*Console) << "Building MissParker project\n";
  BuildMPProject(Project->Name, Project->info.Name,
		 Struct->Modules->List, 
		 (ProjectOptions.MkProjMem!=0));
}

void MainFrame::OnAssemble(wxCommandEvent& WXUNUSED(event)){
#ifdef DEBUG
  fprintf(stderr, "DBG: frame: Entering OnAssemble\n");
#endif
  
  // we must execute the command
  //   <asmdir>/asm <projectname(.axp->.asm)>

  CConfig *Config;
  Config=new CConfig;

  wxArrayString Output, Error;
  wxString Command;
  char asmfile[512];
  int n,i;

  strcpy(asmfile, Project->Name);
  n=strlen(asmfile);
  strcpy(asmfile+n-4, ".asm");
  
#ifndef WIN32
  Command << BIN_INSTALL_DIR << "/";
#endif
  Command << Config->AssemblerDir << "/asm ";
  delete Config;

  if (ProjectOptions.AsmVerbose)
    Command << "-v ";
  Command << "\"";
  Command << asmfile;
  Command << "\"";
  (*Console) << "Executing '" << Command << "'\n";

#ifdef WIN32
  wxExecute(Command, Output, Error);
  n=Output.GetCount();
  for (i=0;i<n;i++)
    (*Console) << Output[i] << "\n";
  n=Error.GetCount();
  for (i=0;i<n;i++)
    (*Console) << Error[i] << "\n";
#else
  wxShell(Command);
#endif

}

void MainFrame::OnMAsm(wxCommandEvent& event){
  OnMake(event);
  OnAssemble(event);
}

void MainFrame::OnProjectOptions(wxCommandEvent& event){
  CProjectOptions POpts(this, -1, &ProjectOptions);
  if (POpts.ShowModal()!=wxID_OK)
    return;

  return;
}

void MainFrame::OnProjectInfo(wxCommandEvent& event){
  CProjectInfo PInfo(this, -1, &(Project->info));
  if (PInfo.ShowModal()==wxID_OK)
    Project->IsModified=1;

  return;
}

void MainFrame::OnSimulate(wxCommandEvent& WXUNUSED(event)){
#ifdef DEBUG
  fprintf(stderr, "DBG: frame: Entering OnSimulate\n");
#endif

  CSimulate Simul(this, -1, Project->Name);
  Simul.ShowModal();

}
  
void MainFrame::OnMissParker(wxCommandEvent& WXUNUSED(event)){
#ifdef DEBUG
  fprintf(stderr, "DBG: frame: Entering OnSimulate\n");
#endif

  CMPPreProjectDialog PreProject(Struct->Modules, this, -1);
  PreProject.ShowModal();

}
  
//LOADvoid MainFrame::OnLoad(wxCommandEvent& event){
//LOAD  char asmfile[255];
//LOAD  int n;

//LOAD  strcpy(asmfile, Project->Name);
//LOAD  n=strlen(asmfile);
//LOAD  strcpy(asmfile+n-4, ".obj");
//LOAD  LoadFile(asmfile, 1);
//LOAD}

// Options Menu
void MainFrame::OnCfgDirs(wxCommandEvent& event){
#ifdef DEBUG
  fprintf(stderr, "DBG: frame: Entering OnCfgDir\n");
#endif

  CConfig *Config;
  Config=new CConfig;

  CDirectories Dirs(this, -1, Config);
  if (Dirs.ShowModal()!=wxID_OK)
    return;

  // update widget
  ModuleList->UpdateList(Dirs.EffectsDir);
  
  delete[] Config->EffectsDir;
  Config->EffectsDir = new char[strlen(Dirs.EffectsDir.c_str())];
  strcpy(Config->EffectsDir, Dirs.EffectsDir.c_str());

  delete[] Config->AssemblerDir;
  Config->AssemblerDir = new char[strlen(Dirs.AssemblerDir.c_str())];
  strcpy(Config->AssemblerDir, Dirs.AssemblerDir.c_str());

  delete[] Config->ProjectDir;
  Config->ProjectDir = new char[strlen(Dirs.ProjectDir.c_str())];
  strcpy(Config->ProjectDir, Dirs.ProjectDir.c_str());

  delete[] Config->SkinDir;
  Config->SkinDir = new char[strlen(Dirs.SkinDir.c_str())];
  strcpy(Config->SkinDir, Dirs.SkinDir.c_str());

  Project->SetOffset(strlen(Config->EffectsDir));

  /*wxString lm;
  lm << "Effects dir set to " << Dirs.EffectsDir.c_str();
  wxLogMessage(lm);
  wxString lm2;
  lm2 << "Assembler dir set to " << Dirs.AssemblerDir.c_str();
  wxLogMessage(lm2);*/
  delete Config;
}

void MainFrame::OnCfgColors(wxCommandEvent& event){
  CThemeSelector ts(this, -1);
  wxString lm;

  if (ts.ShowModal()!=wxID_OK)
    return;
  
  lm << "Using theme: " << ts.NewTheme.Name;

  strcpy(theme.Name  , ts.NewTheme.Name);
  strcpy(theme.TexMod, ts.NewTheme.TexMod);
  strcpy(theme.TexPin, ts.NewTheme.TexPin);
  theme.NameColor[0]=ts.NewTheme.NameColor[0];
  theme.NameColor[1]=ts.NewTheme.NameColor[1];
  theme.NameColor[2]=ts.NewTheme.NameColor[2];
  theme.LinkColor[0]=ts.NewTheme.LinkColor[0];
  theme.LinkColor[1]=ts.NewTheme.LinkColor[1];
  theme.LinkColor[2]=ts.NewTheme.LinkColor[2];
  Struct->Modules->ChangeTheme(&theme);

  Struct->Redraw();

  wxLogMessage(lm);
}

// Help Menu
void MainFrame::OnHelp(wxCommandEvent& event){

}

void MainFrame::OnBugReport(wxCommandEvent& event){

}

void MainFrame::OnTreeDrag(wxTreeEvent& event){
  ModuleList->StartDragItem(event);
}

void MainFrame::OnAbout(wxCommandEvent& WXUNUSED(event)){
  /*  wxString msg;
  msg.Printf( _T("This is the about dialog of minimal sample.\n")
	      _T("Welcome to %s"), wxVERSION_STRING);
  
	      wxMessageBox(msg, "About AGAD", wxOK | wxICON_INFORMATION, this);*/

  CAbout About(this, -1);
  About.ShowModal();
}
