/////////////////////////////////////////////////////////////////////////////
// Name:        glstruct.h
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     05/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#ifndef __PROJ_AL_GLSTRUCT_H__
#define __PROJ_AL_GLSTRUCT_H__

#include <wx/dnd.h>
#include "glmodulelist.h"
#include "Config.h"
#include "fileio.h"
#include "theme.h"

/*
**
** (0, ny) = (-0.5f, 0.5f)                      (nx, ny) = (0.5f, 0.5f)
**    +----------------------------------------------------+
**    |                                   ^                |
**    |                                   |                |
**    |                                   |                |
**    |                                   |                |
**    |                                   |                |
**    |<----------------------------------+--------------->|
**    |             width                 |                |
**    |                                   |                |
**    |                                   |                |
**    |                                   |                |
**    |                            height |                |
**    |                                   |                |
**    |                                   |                |
**    |                                   |                |
**    | [x_p, y_p]                        v                |
**    +----------------------------------------------------+
** (0, 0) = (-0.5f, -0.5f)                      (nx, 0) = (0.5f, -0.5f)
**
**
*/

class CItemDropTarget : public wxTextDropTarget{
public:
    CItemDropTarget(void *i_owner);

    virtual bool OnDropText(wxCoord x, wxCoord y, const wxString& text);

 private:
    void *owner;
    //int IsDrag;
    //char FName[255, ModuleName[255];
};

class CglStruct: public wxScrolledWindow
{
 public:
  CglStruct(wxWindow *parent, TTheme *i_theme,
	    wxGauge *DSPStatus, wxGauge *MemStatus,
	    const wxWindowID id = -1,
	    const wxPoint& pos = wxDefaultPosition, 
	    const wxSize& size = wxDefaultSize,
	    long style = 0, const wxString& name = "Module Structure");
   
  virtual ~CglStruct();
  int NbX, NbY;

  int InitStruct(CFile *i_project);

  void OnPaint(wxPaintEvent& event);
  void OnSize(wxSizeEvent& event);
  //  void OnScroll(wxScrollWinEvent& event);
  void OnMouseEvent(wxMouseEvent& event);

  // right-click on module submenu
  void OnDeleteModule(wxCommandEvent& event);
  void OnPropModule(wxCommandEvent& event);
  void OnNameModule(wxCommandEvent& event);
  void OnInfoModule(wxCommandEvent& event);

  // toggle feedback state for one link
  void OnToggleFeedback(wxCommandEvent& event);

  void Redraw();
  void Render(wxDC *dc);
  void InitDC(wxDC *dc);

  void AddModule(char *Name, char *Path, int xCoord = -1, int yCoord = -1);

  CglModuleList *Modules;
  CLinkList *LLink;
  CFile *Project;
  TTheme *theme;

private:
  wxGauge *DSPStatus;
  wxGauge *MemStatus;
  bool m_init;
  int x_p, y_p, width, height;
  wxMenu *SubMenu;
  wxMenu *LinkSubMenu;
  int tmp_px, tmp_py, tmp_io;
  CItemDropTarget *DropTarget;

DECLARE_EVENT_TABLE()
};

#endif

