#ifndef __PROJ_AL_LINK_K__
#define __PROJ_AL_LINK_K__

#define LINK_OK     0
#define LINK_ERROR  1
#define LINK_DONE   2

#define LINK_MODE_O          0
#define LINK_MODE_I          1
#define LINK_MODE_DELETE_O   2
#define LINK_MODE_DELETE_I   3
#define LINK_MODE_FEEDBACK_I 4
#define LINK_MODE_FEEDBACK_O 5

#define LINK_STATE_NORMAL    0
#define LINK_STATE_FEEDBACK  1

#include "glmodule.h"
#include "theme.h"

typedef struct{  
  int FromCh;
  int ToCh;
  CglModule *From, *To;
  int State;
  void *Next;
} TLink;

class CLinkList{
 public:
  // ctor & dtor
  CLinkList(TTheme *i_theme);
  ~CLinkList();

  // Start a new link if none already started and finish the link
  // started if Status = LINK_STARTED
  // Args are 1. module index, 2. channel, 3. Modo (LINK_MODE_I or LINK_MODE_O)
  // This return LINK_OK if success, LINK_ERROR if some error occured.
  int Process(CglModule*, int, int);

  // Draw links
  int Render(wxDC *dc, int UseTextures=1);

  // create a full link
  int CreateLink(CglModule *From, int FromCh, CglModule *To, int ToCh, int state=LINK_STATE_NORMAL);

  // reset all links
  int Reset();

  // delete all links related to a module
  int DeleteModuleLinks(CglModule*);
  int DeleteModuleLinks(CglModule*, int, int);

  // toggle feedback
  int ToggleFeedback(CglModule*, int, int);

  // update link end
  int SetActiveLinkEnd(int px, int py);

  // create link section
  int CreateLinkSection(FILE *script);

  // read a link section
  int ReadLinkSection(FILE *script, void *mods, int nmod);

  // check if there is a started link
  int IsLinkStarted();

 private:
  TLink *Links;
  int NbLinks;
  int lStatus;                 // LINK_STARTED or LINK_TOSTART
  int ActiveLinkX, ActiveLinkY;
  int CurrentLinkX, CurrentLinkY;
  CglModule *StartModule;
  int StartChannel, Direction;
  TTheme *theme;

  // start a new link
  // arguments are the module and its channel
  int Start(CglModule*, int);

  // finish the start link
  // arguments are the module and its channel
  // if the module index is -1, we cancel the started link.
  int Finish(CglModule*, int, int state=LINK_STATE_NORMAL);
};

#endif
