/////////////////////////////////////////////////////////////////////////////
// Name:        listmodules.cpp
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     05/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#include "listmodules.h"
#include "console.h"
#include <wx/dir.h>
#include <wx/imaglist.h>
#include <wx/bitmap.h>

#include "resources/AgALag/module.xpm"

// ----------------------------------------------------------------------------
// resources
// ----------------------------------------------------------------------------
// the application icon
//#if defined(__WXGTK__) || defined(__WXMOTIF__)
//#include "resources/module.xpm"
//#endif

BEGIN_EVENT_TABLE(CModuleList, wxTreeCtrl)
  EVT_LEFT_DCLICK(CModuleList::OnMouseEvent)
END_EVENT_TABLE()

/*
** This class deals with the modiles list
*/

// ctor & dtor
// ctor last argument is the place where we have to look for modules
CModuleList::CModuleList(wxWindow* parent, wxWindowID id, const wxPoint& pos, 
	      const wxSize& size, const wxString &ModDir, CglStruct *i_glStruct)
  : wxTreeCtrl(parent, id, pos, size, wxTR_HAS_BUTTONS | wxTR_SINGLE){
#ifdef DEBUG
  fprintf(stderr, "DBG: listmodules: Entering constructor\n");
#endif

  // Create image list
  wxBitmap Icon[3];
  Icon[0].LoadFile("../resources/AgALag/module.xpm",wxBITMAP_TYPE_XPM);
  Icon[1].LoadFile("../resources/AgALag/treefolderclosed.xpm",wxBITMAP_TYPE_XPM);
  Icon[2].LoadFile("../resources/AgALag/treefolderopen.xpm",wxBITMAP_TYPE_XPM);
  image = new wxImageList(16, 16, TRUE);
  image->Add(Icon[0]);
  image->Add(Icon[1]);
  image->Add(Icon[2]);
  SetImageList(image);

  Path = ModDir;
  //InsertColumn(0L, "Modules", wxLIST_FORMAT_LEFT, 90);
  GetList();

  glStruct=i_glStruct;
}

CModuleList::~CModuleList(){
#ifdef DEBUG
  fprintf(stderr, "DBG: listmodules: Entering destructor\n");
#endif

  ResetList();
  delete image;
}

// retreive the current selection in the list
int CModuleList::GetSelection(){
#ifdef DEBUG
  fprintf(stderr, "DBG: listmodules: Entering GetSelection\n");
#endif

  return 0;
}

// update the list. if the path given as argument is NULL, we keep the old one 
// Returns MODLIST_OK or MODLIST_ERROR
int CModuleList::UpdateList(const wxString& ModDir){
#ifdef DEBUG
  fprintf(stderr, "DBG: listmodules: Entering UpdateList\n");
#endif

  if (ResetList()!=MODLIST_OK)
    return MODLIST_ERROR;

  if (!ModDir.IsNull()){
    Path=ModDir;
  }

  if (GetList()!=MODLIST_OK)
    return MODLIST_ERROR;

  return MODLIST_OK;
}

// scan modules and subdirs from a given path
// all item found are inserted as child of the given wxTreeItemId parent
int CModuleList::ScanDir(wxString DirStr, wxTreeItemId parent){
  wxDir dir(DirStr);
  wxDir checkdir;
  wxString filename, subdir;
  wxTreeItemId subdirId;
  bool cont;

  if (!dir.IsOpened()){
    // deal with the error here - wxDir would already log an error message
    // explaining the exact reason of the failure
    return MODLIST_ERROR;
  }

  // enumerate subdirs
  cont = dir.GetFirst(&filename, wxEmptyString, wxDIR_DIRS);
  while (cont){
    // only add the directory if it is not empty
    subdir = Path;
    subdir << '/' << filename;
    checkdir.Open(subdir);
    if (checkdir.HasFiles("*.asm")){
      subdirId = AppendItem(parent, filename, 1, 1);
      SetItemImage(subdirId, 2, wxTreeItemIcon_Expanded);
      ScanDir(subdir, subdirId);
    }
    cont = dir.GetNext(&filename);
  };

  // enumerates files in directory
  cont = dir.GetFirst(&filename, MODULE_FILE_PATTERN);
  while (cont){
    AppendItem(parent, filename, 0, 0);
    cont = dir.GetNext(&filename);
  };

  return MODLIST_OK;

}

// fills the list
// Returns MODLIST_OK or MODLIST_ERROR
int CModuleList::GetList(){
#ifdef DEBUG
  fprintf(stderr, "DBG: listmodules: Entering GetList\n");
#endif
  // create root
  wxTreeItemId RootId=AddRoot("Modules", 1, 1);
  int res =  ScanDir(Path, RootId);

  SetItemImage(RootId, 2, wxTreeItemIcon_Expanded);
  Expand(RootId);

  return res;
}

// reset the list
// Returns MODLIST_OK or MODLIST_ERROR
int CModuleList::ResetList(){
#ifdef DEBUG
  fprintf(stderr, "DBG: listmodules: Entering ResetList\n");
#endif

  DeleteAllItems();

  return MODLIST_OK;
}

// handle double click mouse events
void CModuleList::OnMouseEvent(wxMouseEvent& event){
  char FName[255], tmp[255];
  wxTreeItemId RootItem = GetRootItem();
  wxTreeItemId parent;
  wxString lm;
  TModuleInfo info;
  CConfig *cfg;
  
  // get selected item
  wxTreeItemId SelId = HitTest(event.GetPosition());

  if (!SelId){
    return;
  }

  if (ItemHasChildren(SelId)){
    event.Skip();
    return;
  }

  strcpy(FName,"");
  parent = GetItemParent(SelId);
  while (parent!=RootItem){
    strcpy(tmp, FName);
    sprintf(FName, "%s/%s", (GetItemText(parent)).GetData(), tmp);
    parent = GetItemParent(parent);
  };


  cfg = new CConfig;
  sprintf(tmp,"%s/%s%s",cfg->EffectsDir,FName,(GetItemText(SelId)).GetData());
  if (GetModuleInfo(tmp, &info)==MODULE_HEADER_ERROR){
    wxLogMessage("Module doesn't fit the template, no information can be given.");
  } else {
    lm << "Module: " << FName << (GetItemText(SelId)).GetData() 
       << "\n    Name: " << info.Name
       << "\n    Description: " << info.Descr
       << "\n    " << info.NbIn << " inputs, " << info.NbOut << " outputs"
       << "\n    Inputs: " << info.InDescr
       << "\n    Outputs: " << info.OutDescr;
    wxMessageBox(lm, "Information");
  }
  delete cfg;
}

int CModuleList::StartDragItem(wxTreeEvent event){
  char FName[255], ModuleName[255], tmp[512];
  wxTreeItemId RootItem = GetRootItem();
  wxTreeItemId parent;

  wxTreeItemId SelId = event.GetItem();

  if (!SelId){
    return 1;
  }

  if ((ItemHasChildren(SelId)) || (GetItemImage(SelId)!=0)){
    return 1;
  }

  strcpy(FName,"");
  parent = GetItemParent(SelId);
  while (parent!=RootItem){
    strcpy(tmp, FName);
    sprintf(FName, "%s/%s", (GetItemText(parent)).GetData(), tmp);
    parent = GetItemParent(parent);
  };

  strcpy(tmp,(GetItemText(SelId)).GetData());

  // remove ".asm". We are thus left only with the module name.
  strncpy(ModuleName, tmp, strlen(tmp)-4);
  ModuleName[strlen(tmp)-4]=0;

  //glStruct->AddModule(ModuleName, FName);
  sprintf(tmp, "%s$%s", ModuleName, FName);
  wxTextDataObject ModuleData(tmp);
  wxDropSource DropModule(this, wxDROP_ICON(module));
  DropModule.SetData(ModuleData);
  DropModule.DoDragDrop(TRUE);

  return 0;
}
