/////////////////////////////////////////////////////////////////////////////
// Name:        listmodules.h
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     05/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#ifndef __PROJ_AL_LIST_MODULES_H__
#define __PROJ_AL_LIST_MODULES_H__

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/treectrl.h>

#include "glstruct.h"

// defines the extension of files we will list
#define MODULE_FILE_PATTERN "*.asm"

#define MODLIST_OK    0
#define MODLIST_ERROR 1

/*
** This class deals with the modiles list
*/

class CModuleList : public wxTreeCtrl {
 public:
  // ctor & dtor
  // ctor last argument is the place where we have to look for modules
  CModuleList(wxWindow* parent, wxWindowID id, const wxPoint& pos, 
	      const wxSize& size, const wxString& ModDir,CglStruct *i_glStruct);

  ~CModuleList();

  // retreive the current selection in the list
  int GetSelection();

  // update the list. if the path given as argument is NULL, we keep the old one 
  // Returns MODLIST_OK or MODLIST_ERROR
  int UpdateList(const wxString& ModDir);

  // start a dragging event
  int StartDragItem(wxTreeEvent event);

 private:

  // scan modules and subdirs from a given path
  // all item found are inserted as child of the given wxTreeItemId parent
  int CModuleList::ScanDir(wxString Path, wxTreeItemId parent);

  // fills the list
  // Returns MODLIST_OK or MODLIST_ERROR
  int GetList();

  // reset the list
  // Returns MODLIST_OK or MODLIST_ERROR
  int ResetList();

  // handle double click mouse events
  void OnMouseEvent(wxMouseEvent& event);

  //long nb;            // number of modules
  wxString Path;      // path for module search.
  wxImageList *image; // image for the items

  CglStruct *glStruct; // opengl canvas

  DECLARE_EVENT_TABLE()
};



#endif
