/////////////////////////////////////////////////////////////////////////////
// Name:        mpproject.h
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.0
// Created:     12/03/04
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#ifndef __MP_PROJECT_H__
#define __MP_PROJECT_H__

// For compilers that support precompilation, includes "wx/wx.h".
//#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include <wx/treectrl.h>
#include <wx/listctrl.h>
#include <wx/proplist.h>
#include <wx/propform.h>
#include <wx/checkbox.h>
#include <wx/imaglist.h>
#include <wx/notebook.h>

#include "glmodulelist.h"
#include "mpprojectfile.h"

class CMPPreProjectDialog;

class CParameterTree : public wxTreeCtrl{
 public:
  CParameterTree(wxWindow *i_parent, wxWindowID id, const wxPoint& pos, 
		 const wxSize& size);
  ~CParameterTree();

  int GetParamList(TglModuleList*);
  int AddParam(TParameterItem *prop, wxTreeItemId mod_id);

  wxImageList *image;
};

class CParameterManager : public wxWindow{
 public:
  CParameterManager(wxWindow *i_parent, wxWindowID id, const wxPoint& pos, 
		    const wxSize& size);
  ~CParameterManager();

  int AppendGroup(wxString group);

  CMPPreProjectDialog *parent;

  CParameterTree *param_tree;
  wxCheckBox *enable;
  wxCheckBox *TxtAssign;
  wxComboBox *GroupAssign;

  //DECLARE_EVENT_TABLE()
};  

class CGroupManager : public wxWindow{
 public:
  CGroupManager(wxWindow *i_parent, wxWindowID id, const wxPoint& pos, 
		    const wxSize& size);
  ~CGroupManager();

  void OnGroupAdd(wxCommandEvent &event);
  void OnGroupRemove(wxCommandEvent &event);

  CMPPreProjectDialog *parent;

  wxTreeCtrl *group_tree;
  wxTextCtrl *ToAdd;
  wxButton *Add;
  wxButton *Remove;

  DECLARE_EVENT_TABLE()
};  

class CMPPreProjectDialog : public wxDialog{
 public:
  // ctor & dtor
  CMPPreProjectDialog(CglModuleList *i_mods, wxWindow* parent, wxWindowID id);
  ~CMPPreProjectDialog();

  CglModuleList *l_mods;
  TglModuleList *mods;
  TParameterItem *groups;
  CParameterManager *page_param;
  CGroupManager *page_group;

  int UpdateControls(TParameterItem *);

 private:
  void OnSelChanged(wxCommandEvent& event);
  void OnGroupChanged(wxCommandEvent& event);
  void OnCompChanged(wxCommandEvent& event);
  void OnTypeChanged(wxCommandEvent& event);
  void OnStore(wxCommandEvent &event); 
  void OnAddToCombo(wxCommandEvent &event);
  void OnRemoveCombo(wxCommandEvent &event);
  void OnDefaultCombo(wxCommandEvent &event);
  void OnEnable(wxCommandEvent &event);
  void OnAssignGroup(wxCommandEvent &event);
  void OnChangeAssignGroup(wxCommandEvent &event);

  int SetComboDefault(bool flag=true);
  int EnableControls(bool flag);

  wxButton *BtnOk, *BtnCancel;
  wxButton *BtnStore;
  CParameterTree *tree;
  //wxTreeCtrl *group_tree;
  wxNotebook *pager;
  TParameterItem *actual_prop;
  wxString CompChoices[2];
  wxString TypeChoices[2];
  wxString ProgChoices[3];
  wxString DispChoices[2];

  wxCheckBox *EnableParam;
  wxStaticText *TxtIntro, *TxtDescr,
    *TxtDisplay, *TxtActual, *TxtMin, *TxtMax, *TxtVal,
    *TxtComp, *TxtType, *TxtProg, *TxtNbSteps, *TxtDisp,
    *TxtTableName;
  wxTextCtrl *EdtDescr, 
    *EdtMin, *EdtMax, *EdtDVal, *EdtDMin, *EdtDMax, *EdtNbSteps,
    *EdtTableName,
    *EdtComboLabelToAdd, *EdtComboValueToAdd;
  wxComboBox *CboComp,*CboType,*CboProg,*CboDisp;
  wxListCtrl *ComboTable;
  wxButton *AddToCombo, *RemoveFromCombo, *DefaultCombo;

  DECLARE_EVENT_TABLE()
};

int LoadMPProject(char *fname, TglModuleList *mods);
int BuildMPProject(char *fname, char *name, TglModuleList *mods, bool map);

#endif
