/////////////////////////////////////////////////////////////////////////////
// Name:        projopt.cpp
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.0
// Created:     02/03/03
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#include "projopt.h"

// ID's
#define ID_STATIC         1
#define ID_ASM_VERBOSE    2
#define ID_MKPROJ_VERBOSE 3
#define ID_MKPROJ_MAP     4

#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/valgen.h>

//-----------------------------------------------------
// implementation of CAbout
//-----------------------------------------------------

CProjectOptions::CProjectOptions(wxWindow* parent, wxWindowID id, TProjectOptions *po)
  : wxDialog(parent, id, "Options", wxDefaultPosition, wxSize(250,170), wxDEFAULT_DIALOG_STYLE){
#ifdef DEBUG
  fprintf(stderr, "DBG: directories: Entering constructor\n");
#endif
  wxString MkProjVerboseChoices[3] = {wxString("none"), wxString("simple"), wxString("full")};

  SetClientSize(300,260);
 
  // creates all widgets inside the dialog box
  TxtIntro = new wxStaticText(this, ID_STATIC, "Specify options for project generation" , wxPoint( 15, 10), wxSize(210, 20));
 
  TxtAsm = new wxStaticText(this, ID_STATIC, "Assembler:", wxPoint( 15, 40), wxSize(210, 20));
  cbAsmVerbose = new wxCheckBox(this, ID_ASM_VERBOSE, "verbose", 
				wxPoint(25, 60), wxSize(200, 20),
				0, wxGenericValidator(&po->AsmVerbose));

  if (po->AsmVerbose){
    cbAsmVerbose->SetValue(1);
  } else {
    cbAsmVerbose->SetValue(0);
  }    
 
  TxtMkProj = new wxStaticText(this, ID_STATIC, "Project Generator Options:", wxPoint( 15,100), wxSize(210, 20));
  rbMkProjVerbose = new wxRadioBox(this, ID_MKPROJ_VERBOSE, "verbose level",
				   wxPoint(25,130), wxSize(100,90),
				   3, MkProjVerboseChoices, 1,
				   wxRA_SPECIFY_COLS, wxGenericValidator(&po->MkProjVerboseLevel));
  cbMkProjLab = new wxCheckBox(this, ID_ASM_VERBOSE, "generate label file",
			       wxPoint(135, 140), wxSize(150, 25),
			       0, wxGenericValidator(&po->MkProjLab));
  cbMkProjMem = new wxCheckBox(this, ID_ASM_VERBOSE, "generate memory map",
			       wxPoint(135, 165), wxSize(150, 25),
			       0, wxGenericValidator(&po->MkProjMem));
  
  if ((po->MkProjVerboseLevel<0) || (po->MkProjVerboseLevel>2)) po->MkProjVerboseLevel=0;
  rbMkProjVerbose->SetSelection(po->MkProjVerboseLevel);

  if (po->MkProjMem){
    cbMkProjMem->SetValue(1);
  } else {
    cbMkProjMem->SetValue(0);
  }

  if (po->MkProjLab){
    cbMkProjLab->SetValue(1);
  } else {
    cbMkProjLab->SetValue(0);
  }

  BtnOk = new wxButton(this, wxID_OK, "Ok", wxPoint(140, 230), wxSize(50, 22));
  BtnCancel = new wxButton(this, wxID_CANCEL, "Cancel", wxPoint(220, 230), wxSize(50, 22));
}

CProjectOptions::~CProjectOptions(){
#ifdef DEBUG
  fprintf(stderr, "DBG: directories: Entering destructor\n");
#endif
}
