//  AgALag : Axoris group AL3101 application generator                //
//                                                                    //
//                 _         _    _      _                            //
//                / \____   / \  | |    / \____                       //
//               / _ \  _\ / _ \ | |   / _ \  _\                      //
//              / /_\ \  __ /_\ \| |  / /_\ \  __                     //
//             / _____ \ \ \____ \ |_/______ \ \ \                    //
//            /_/     \_\___\   \_\____|    \_\___\                   //
//                                                                    //
//  File: main.cpp                                                    //
//  Description: application entry point                              //
//  Version: 1.0                                                      //
//  Author: Gregor                                                    //
//  e-mail: Gregor.Soyez@swing.be                                     //
//                                                                    //
//  Copyright (C) 2002 Axoris                                         //
//--------------------------------------------------------------------//

#include <stdio.h>
#include "structure.h"
#include "messages.h"

#ifdef WIN32
#include "../common/option.h"
#else
#include <unistd.h>
#endif
#include "options.h"

TOptions Options;

void PrintHelp()
{
  OUTPUT("Application generator for AL3101 by Axoris Group (AgALag)\n");
  OUTPUT("\tUsage : ./mkalproj <options> project_filename\n");
  OUTPUT("\t  -h : print this help message\n");
  OUTPUT("\t  -l : generate a label file\n");
  OUTPUT("\t  -t : generate a memory offset table\n");
  OUTPUT("\t  -m : consider an extern a memory map\n");
  OUTPUT("\t  -v : verbose mode\n");
  OUTPUT("\t  -V : Be VERY verbose\n");
  OUTPUT("Please send Bug Reports to gregor.soyez@swing.be\n\n");
}

int main(int argc, char *argv[]){

  char optdescr;

  Options.Verbose=Options.VeryVerbose=Options.GenerateMem=Options.GenerateLab=Options.GenerateTable=0;
  // retreive options:
  while ((optdescr = getopt(argc, argv, "hvVmlt")) != -1){
    switch (optdescr){
    case 'h': PrintHelp(); return 0; break;
    case 'v': Options.Verbose = 1; break;
    case 'V': Options.Verbose = Options.VeryVerbose = 1; break;
    case 'm': Options.GenerateMem = 1; break;
    case 'l': Options.GenerateLab = 1; break;
    case 't': Options.GenerateTable = 1; break;
    case ':': PrintHelp(); ERROR("Missing argument. Giving up\n"); return 1; break;
    case '?': PrintHelp(); ERROR("Unknown option. Giving up\n"); return 1; break;
    default: return 1; break;
    };
  }

  if ((argc==1) ||(argv[argc-1][0]=='-')){
    PrintHelp();
    return 1;
  }

  MakeApplication(argv[argc-1]);

  return 0;
}
