//--------------------------------------------------------------------//
//  AgALag : Axoris group AL3101 application generator                //
//                                                                    //
//                 _         _    _      _                            //
//                / \____   / \  | |    / \____                       //
//               / _ \  _\ / _ \ | |   / _ \  _\                      //
//              / /_\ \  __ /_\ \| |  / /_\ \  __                     //
//             / _____ \ \ \____ \ |_/______ \ \ \                    //
//            /_/     \_\___\   \_\____|    \_\___\                   //
//                                                                    //
//  File: memory.cpp                                                  //
//  Description: functions dealing with memory offsets                //
//  Version: 1.0                                                      //
//  Author: Gregor                                                    //
//  e-mail: Gregor.Soyez@swing.be                                     //
//                                                                    //
//  Copyright (C) 2002 Axoris                                         //
//--------------------------------------------------------------------//
#ifndef __PROJ_AL_MEMORY_H__
#define __PROJ_AL_MEMORY_H__

#include "ProjAL.h"
#include <stdio.h>
#include "messages.h"

int __MemorySegmentAllocationPtr;

/* 
** initialize the memory allocating stuff
*/
int MemAllocInit(){
  __MemorySegmentAllocationPtr=0;
  return 0;
}

/*
** the following function retreive an offset
** for a memory segment of given length
** It allocates memory starting from the beginning
** thus if some delay-line requires the first memory
** segment, you have to call the SegAlloc function
** for that segment before any other one
** It returns the offset if successful and 
** -1 if there is not enough memory
*/
int SegAlloc(int Size){
  if (Size<0){
    WARNING("SegAlloc: Requesting negative memory segment has no effect\n");
    return __MemorySegmentAllocationPtr;
  }

  if (1023-__MemorySegmentAllocationPtr<Size){
    ERROR2("SegAlloc: Not enough free memory space (%d instead of %d)\n", 1023-__MemorySegmentAllocationPtr, Size);
    return -1;
  }

  __MemorySegmentAllocationPtr+=Size;
  return __MemorySegmentAllocationPtr-Size;
}

#endif
