//--------------------------------------------------------------------//
//  AgALag : Axoris group AL3101 application generator                //
//                                                                    //
//                 _         _    _      _                            //
//                / \____   / \  | |    / \____                       //
//               / _ \  _\ / _ \ | |   / _ \  _\                      //
//              / /_\ \  __ /_\ \| |  / /_\ \  __                     //
//             / _____ \ \ \____ \ |_/______ \ \ \                    //
//            /_/     \_\___\   \_\____|    \_\___\                   //
//                                                                    //
//  File: tokenize.cpp                                                //
//  Description: tool for string tokenization                         //
//  Version: 1.0                                                      //
//  Author: Gregor                                                    //
//  e-mail: Gregor.Soyez@swing.be                                     //
//                                                                    //
//  Copyright (C) 2002 Axoris                                         //
//--------------------------------------------------------------------//

#include <string.h>
#include <stdio.h>
#include "tokenize.h"

/*
** This function tokenize the string Str. It returns 
** the number of tokens. It allocates and fills the tokens 
** array. A pointer to that array is returned through ptoken.
 */
int StrTokenize(char *str, char ***ptokens){
  char **tokens;
  int start, end;
  int len, i, nb;

  // retreive the length of the command line
  len=strlen(str);

  // if CommandLine is an emtpy string, return 0
  if (len==0)
    return 0;

  // if there is(are) EOL caracter(s), replace it(them) by null caracters
  // We are taking into account two different types of EOL caracters :
  //   0x0A and 0x0D.
  // For windows, we should have both while under Linux only one of them 
  // should be present
  while ((str[len-1]==0x0A) || (str[len-1]==0x0D)){
    str[len-1]='\0';
    len--;
    // if CommandLine is an emtpy string, return 0
    if (len==0)
      return 0;
  }
  
  nb=0;

  // count firstly the number of tokens
  start=0;
  do{   // we loop until we arrive at EOL
    // find the beginning of the next token
    while ((str[start]=='\t') || (str[start]==' ')) start++;

    end=start+1;

    // if EOL, stop counting else, find end of current token
    if (start<len){
      // scan the beginning token ...
      while ((str[end]!='\t') && (str[end]!=' ') && (end<len)) end++;
      
      // ... and count it
      nb++;
    }
    
    // prepare for a next search.
    start=end+1;
  } while (start<=len);

  // stop here if thhere is no token
  if (nb==0) return 0;

  // allocate momory for the tokens array
  tokens=new char *[nb];

  // copy all tokens to the array
  // we COPY the tokens without just make the array ellements point to
  // some positions in the string

  start=0;
  i=0;
  do{   // we loop until we arrive at EOL
    // find the beginning of the next token
    while ((str[start]=='\t') || (str[start]==' ')) start++;

    // if EOL, return the number of tokens found
    if (start==len){
      // point ptokens to tokens
      (*ptokens)=tokens;
      return nb;
    }

    // scan the beginning token ...
    end=start+1;
    while ((str[end]!='\t') && (str[end]!=' ') && (end<len)) end++;

    // ... and store it.
    tokens[i]=new char[end-start+1];
    strncpy(tokens[i], str+start, end-start);
    tokens[i][end-start]='\0';
    i++;

    // prepare for a next search.
    start=end+1;
  } while (start<=len);

  // point ptokens to tokens
  (*ptokens)=tokens;

  return i; // returns the number of tokens found
}
