#include "Format.h"
#include <stdio.h>
#include <string.h>

// Extract the Hexadecimal value from a string. 
// Return CST_OK if success, CST_BADFORMAT otherwise
int ReadHexa(char *Str, int len, UINT32 *c){
  int i;
  char s[255];

  for (i=0;i<len;i++) 
    if ((Str[i]<'0') 
	|| ((Str[i]>'9') && (Str[i]<'A'))
	|| (Str[i]>'F'))
      return CST_BADFORMAT;
  sprintf(s, "0x%s", Str);
  sscanf(s, "%i", c);
  return CST_OK;
}

// Extract the Octal value from a string. 
// Return CST_OK if success, CST_BADFORMAT otherwise
int ReadOctal(char *Str, int len, UINT32 *c){
  int i;
  char s[255];

  for (i=0;i<len;i++) 
    if ((Str[i]<'0') || (Str[i]>'7')) return CST_BADFORMAT;
  sprintf(s, "0%s", Str);
  sscanf(s, "%i", c);
  return CST_OK;
}

// Extract the Binary value from a string. 
// Return CST_OK if success, CST_BADFORMAT otherwise
int ReadBinary(char *Str, int len, UINT32 *c){
  int i, a;

  (*c) = 0;
  a=1;

  for (i=len-1;i>=0;i--) 
    switch (Str[i]){
      case '1': 
	(*c)+=a;   // no break needed;
      case '0': 
	a<<=1;
	break;
      default:
	return CST_BADFORMAT;
    }
  return CST_OK;
}

// Extract the decimal value from a string. 
// Return CST_OK if success, CST_BADFORMAT otherwise
// If nmant>0 we request a format S3.nmant
// nmant<=0 requires a integer value.
int ReadDecimal(char *Str, int len, UINT32 *c, int nmant){
  int i, IsPoint, Sign;

  IsPoint = 0;
  Sign = 1;

  for (i=0;i<len;i++){
    switch (Str[i]){
    case '0': case '1': case '2': case '3': case '4': 
    case '5': case '6': case '7': case '8': case '9':
      break;
    case '-': 
      Sign=-1;   // no break; needed
    case '+':
      if (i!=0) return CST_BADFORMAT;
      break;
    case '.':
      IsPoint = 1;
      break;
    default:
      return CST_BADFORMAT; 
    }
  }

  if (IsPoint==0){
    // The following line must be uncommented if th purely integer case
    // is not allowed for S3.x formats
    // This means that 1 must be writed 1.0 !
    if (nmant>0) return CST_BADFORMAT;
    // Integer values are never negatives.
    if (Sign==-1) return CST_BADFORMAT;
    sscanf(Str, "%d", c);
  } else {
    double f;
    if (nmant<=0) // we expect a purely integer value so float not allowed
      return CST_BADFORMAT;
    if (Sign==-1){
      sscanf(Str+1, "%lf", &f);
      f=16.0-f;
    } else {
      sscanf(Str, "%lf", &f);
    }
    (*c) = (INT32) (f*(1<<nmant));
  }

  return CST_OK;
}


// extract hexadecimal constant from a given string
// the allowed formats are 
//     -  0X...   Hexadecimal format
//     -  $...    Hexadecimal format
//     -  ...h    Hexadecimal format
//     -  %...    Binary format
//     -  ...B    Binary format
//     -  ...Q    Octal format
//     -  Integer and floating point formats.
// returns 0 if success, 1 if an error is detected in the constant format
// if the function succeed, the constant is stored in 'cst'
// 'nmant' contains the expectes number of bits after the decimal point.
// if nmant<=0, the constant has to be an integer
// (decimal or hexadecimal format).
// if nmant==-1 (NMANT_ADDR), we directly ask for an address 
// (i.e. we call preasm with CST_ABS when required)
// if nmant==-2 (NMANT_LABEL), we allow for a CST_EQU or a Label

int ExtractHexa(char *String, UINT32 *cst, int nmant){
  int i, len, sign;
  UINT32 Term1, Term2;
  int PlusPosition;
  char WorkString[255];
  int PreAsmType;       // in case of a call to the preasm ! 

  // length of the parameter string
  len = strlen(String);

  // construct the Type to pass to the preasm from nmant
  if (nmant==NMANT_LABEL)
    PreAsmType=CST_LABEL;
  else if (nmant==NMANT_ADDR)
    PreAsmType=CST_ABS;
  else
    PreAsmType=CST_EQU;

  // Check the format of the constant.
  // The correct caracters are 
  //   - Letters : Q,B,X,H defines format
  //               Used in a pre-defined value
  //   - Numbers
  //   - '.' for floating point values
  //   - '-' for negative values
  //   - '%', '$' defines binary and hexadecimal values
  //   - '+' defines a positive constant if in first position
  //         defines a sum otherway : in ths case, format must be
  //            'pre-defined constant'+constant

  // step 1: check the '+' syntax.
  PlusPosition = -1;
  for (i=0;i<len;i++)
    if (String[i]=='+'){
      if (PlusPosition!=-1)
	return CST_BADFORMAT;
      PlusPosition=i;
    }
  if (PlusPosition>0){   // the case PlusPosition==0 defines constant format !
    char PredefCst[255];
    strncpy(PredefCst, String, PlusPosition);
    PredefCst[PlusPosition]='\0';
    if (PreAsmGetConstant(PredefCst, WorkString, PreAsmType) !=0){
      return CST_UNDEFINED;
    }
    if ((sign = ExtractHexa(WorkString, &Term1, nmant))!=CST_OK)
      return sign;
    if ((sign = ExtractHexa(String+PlusPosition+1, &Term2, nmant))!=CST_OK)
      return sign;
    (*cst) = Term1+Term2;
    return CST_OK;
  }

  // step 2 : determine format and call ad hoc function to read constant

  if (String[0]=='%'){      // Binary format
    return ReadBinary(String+1, len-1, cst);
  }

  if (String[0]=='$') {  // hexadecimal format
    return ReadHexa(String+1, len-1, cst);
  }

  if ((String[0]=='0') && (String[1]=='X')){  // hexadecimal format
    return ReadHexa(String+2, len-2, cst);
  }

  if (String[len-1]=='B'){  // second binary format
    // Note: the case of hexadecimal values starting with 0x or $ and 
    // finishing with B has already been treated
    if ((String[0]>='A') && (String[0]<='Z')){  // predefined constant
      if (PreAsmGetConstant(String, WorkString, PreAsmType) !=0){
	return CST_UNDEFINED;
      }
      return ExtractHexa(WorkString, cst, nmant); 
    }
    strncpy(WorkString, String, len-1);
    WorkString[len-1]='\0';
    return ReadBinary(WorkString, len-1, cst);
  }

  if (String[len-1]=='Q') {  // octal format
    if ((String[0]>='A') && (String[0]<='Z')){  // predefined constant
      if (PreAsmGetConstant(String, WorkString, PreAsmType) !=0){
	return CST_UNDEFINED;
      }
      return ExtractHexa(WorkString, cst, nmant); 
    }
    strncpy(WorkString, String, len-1);
    WorkString[len-1]='\0';
    return ReadOctal(WorkString, len-1, cst);
  }

  if (String[len-1]=='H') {  // hexadecimal format
    if ((String[0]>='A') && (String[0]<='Z')){  // predefined constant
      if (PreAsmGetConstant(String, WorkString, PreAsmType) !=0){
	return CST_UNDEFINED;
      }
      return ExtractHexa(WorkString, cst, nmant); 
    }
    strncpy(WorkString, String, len-1);
    WorkString[len-1]='\0';
    return ReadHexa(WorkString, len-1, cst);
  }

  if (((String[0]>='0') && (String[0]<='9'))  // decimal (or float) format
      || (String[0]=='+') || (String[0]=='-')){
    return ReadDecimal(String, len, cst, nmant);
  }

  if ((String[0]>='A') && (String[0]<='Z')){  // predefined constant
    if (PreAsmGetConstant(String, WorkString, PreAsmType) !=0){
      return CST_UNDEFINED;
    }
    return ExtractHexa(WorkString, cst, nmant); 
  }
  
  // if we arrive here, constant matches no accepted format !
  return CST_BADFORMAT;
}
