#ifndef __ASM_FUNC_H__
#define __ASM_FUNC_H__

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "disasmfunc.h"

int ConstantFormat;

// conditions
char* DisasmConditionTable[] = {
  "!Z",
  "Z",
  "!V",
  "V",
  "N",
  "!N",
  "VN",
  "V!N",
  "!VN",
  "!V!N",
  "!V!N!Z",
  "!N!Z",
  "N+Z",
  "!V(N+Z)",
  "",
  "VZ"
};

int SetConstantFormat(int format){
  ConstantFormat = format;

  return 0;
}

/*
 * Converts a constant c (length givent by bits after decimal point) 
 * to a string s.
 * The format is hexadecimal by default and decimal if
 * disasm called with the -d option
 */
int Constant2String(UINT32 c, int bits, char *s){
  if (ConstantFormat == CST_FORMAT_DECIMAL){
    if (bits==0)
      sprintf(s, "%d", c);
    else{
      double f = (double) c;
      while (bits--) f*=0.5;
      sprintf(s, "%f", f>=8.0 ? f-16.0 : f);
    }
  } else {
    sprintf(s, "0x%x", c);
  }

  return 0;
}

// DISASSEMBLING FUNCTIONS
// all the assembling functions have the same structure :
//   they take the 2 higher bits, the 32 lower bits, and a pointer to 
//   a string in entry. The returned value is 0 if sucess or the -index of
//   the failing parameter. Disassembled string is returned through the
//   pointer passed as parameter. If the mnemonic requires an extrabit, the
//   extrabit is returned.
//   The string is assumed to be allocated !
 
// normal mnemonics (no extrabit, no condition)
int disasm_c18_x_m(UINT32 wparam, UINT32 lparam, char *out){
  UINT32 c18 = (wparam<<20) + (lparam>>12);
  UINT32 m = (lparam%0x800);
  char s1[40];
  char s2[40];

  Constant2String(c18,18,s1);
  Constant2String(m,0,s2);

  sprintf(out, "%s %s", s1, s2);
  return 0;
}

int disasm_x_m(UINT32 wparam, UINT32 lparam, char *out){
  UINT32 m = (lparam%0x800);

  Constant2String(m, 0, out);
  return 0;
}

int disasm_c24_x(UINT32 wparam, UINT32 lparam, char *out){
  UINT32 c24 = (wparam<<26) + (lparam>>6);

  Constant2String(c24, 24, out);
  return 0;
}

int disasm_c18_x(UINT32 wparam, UINT32 lparam, char *out){
  UINT32 c18 = (wparam<<20) + (lparam>>12);

  Constant2String(c18, 18, out);
  return 0;
}

int disasm_c18_d8(UINT32 wparam, UINT32 lparam, char *out){
  UINT32 c18 = (wparam<<20) + (lparam>>12);
  UINT32 d8  = (lparam%0x1000);
 
  char s1[40];
  char s2[40];

  Constant2String(c18,18,s1);
  Constant2String(d8,8,s2);

  sprintf(out, "%s %s", s1, s2);
  return 0;
}

int disasm_c8_d18(UINT32 wparam, UINT32 lparam, char *out){
  UINT32 c8  = (wparam<<10) + (lparam>>22);
  UINT32 d18 = (lparam%0x400000);

  char s1[40];
  char s2[40];

  Constant2String(c8,8,s1);
  Constant2String(d18,18,s2);

  sprintf(out, "%s %s", s1, s2);
  return 0;
}

int disasm_x(UINT32 wparam, UINT32 lparam, char *out){
  sprintf(out, "");
  return 0;
}

// mnemonics with condition
int disasm_c24_x_p(UINT32 wparam, UINT32 lparam, char *out){
  UINT32 c24 = (wparam<<26) + (lparam>>6);
  UINT32 cdt = (lparam%0x10);

  char s[40];
  Constant2String(c24, 24, s);

  sprintf(out, "%s %s", DisasmConditionTable[cdt], s);
  return 0;
}

int disasm_c8_x_p(UINT32 wparam, UINT32 lparam, char *out){
  UINT32 c8  = (wparam<<6) + (lparam>>26);
  UINT32 cdt = (lparam%0x10);

  char s[40];
  Constant2String(c8, 0, s);

  sprintf(out, "%s %s", DisasmConditionTable[cdt], s);
  return 0;
}

// mnemonics with extra bit
int disasm_c10_x_b_m(UINT32 wparam, UINT32 lparam, char *out){
  int ExtraBit = (int) ((lparam>>11)%0x2);
  UINT32 c10 = (wparam<<8) + (lparam>>24);
  UINT32 m   = (lparam%0x800);

  char s1[40];
  char s2[40];

  Constant2String(c10, 0, s1);
  Constant2String(m, 0, s2);

  sprintf(out, "%s %s", s1, s2);
  return ExtraBit;
}

int disasm_c24_b_x(UINT32 wparam, UINT32 lparam, char *out){
  int ExtraBit = (int) ((lparam>>5)%0x2);
  UINT32 c24;
  
  if (ExtraBit){
    c24 = (wparam<<26) + (lparam>>6);
    Constant2String(c24, 24, out);
  } else {
    sprintf(out, "");
  }
  return ExtraBit;
}

#endif
