/********************************************************************/
/*                                                                  */
/*                        A X O R I S                               */
/*                                                                  */
/*                Alesis AL3101/2 DSP Simulator                     */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Origin Date    : June 2001                                       */
/* Project Version: v0.1                                            */
/* FileName       : AL3101_Core.c                                   */
/* File Version   : v0.1                                            */
/* Author         : Fabian Duchesne                                 */
/* Last Modif     : September 2001                                  */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/*       G L O B A L   I N C L U D E								*/
/********************************************************************/
#include <stdio.h>
//#include <windows.h>

/********************************************************************/
/*       W X   W i n   I N C L U D E								*/
/********************************************************************/
//#include <wx/wx.h>
#include <wx/thread.h>
#include <wx/utils.h>

#if !wxUSE_THREADS
    #error "This sample requires thread support!"
#endif // wxUSE_THREADS

/********************************************************************/
/*       P o r t A u d i o  I N C L U D E							*/
/********************************************************************/
#ifdef _STREAMING_ENABLED_
#include "portaudio.h"
#endif

/********************************************************************/
/*       L O C A L   I N C L U D E									*/
/********************************************************************/
#include "./../MessageMacros.h"
#include "AL3101_Core.h"
#include "./../1K_func/AL3101Table.h"
#include "./../Debug.h"
#include "./../Error.h"
#include "./../FileIO/FileIO_Generic.h"		// for the FileIO errors
#include "./../HelperFunctions.h"

/********************************************************************/
/*       G L O B A L   V A R I A B L E S							*/
/********************************************************************/
bool		ContinueSimulationFlag	= TRUE;
bool		SimulationRunning = FALSE;
INT32		IsStreaming = 0;
//wxMutex		SafeBoolAccess;
wxCondition	*LaunchSimulation;
wxMutex		LaunchSimulationMutex;
/*wxCondition	*LaunchSimulationResponse;
wxMutex		LaunchSimulationResponseMutex;*/
INT32		ThreadStarted = FALSE;
wxCondition *SimulationFinished;
wxMutex		SimulationFinishedMutex;
INT32		NumberOfSteps;
extern TMnemonicTable MnemonicTable[];
extern const CHAR* PROMPT_1KSM;

#include <wx/listimpl.cpp>
WX_DEFINE_LIST(ValBreakpoint_List);
WX_DEFINE_LIST(RegisterWatch_List);
WX_DEFINE_LIST(MemoryWatch_List);
WX_DEFINE_LIST(CommandAlias_List);

/********************************************************************/
/*       H E L P E R   F U N C   P R O T O T Y P E S				*/
/********************************************************************/
inline INT32 AL3101_LoadDataFromInputs(AL3101 *pDspInstance);
inline INT32 AL3101_LoadDataFromSignals(AL3101 *pDspInstance);
inline INT32 AL3101_LoadDataFromTools(AL3101 *pDspInstance);
inline INT32 AL3101_SaveDataToOutputs(AL3101 *pDspInstance);
inline INT32 AL3101_SaveDataToTraces(AL3101 *pDspInstance);
inline INT32 AL3101_SaveDataToTools(AL3101 *pDspInstance);
inline INT32 AL3101_CheckBreakpoints(AL3101* pDspInstance);
inline void AL3101_DisplayAssemblyLine(AL3101* pDspInstance);

extern INT32 CmdDelAllBreakPoints(CHAR* Options);
extern INT32 CmdDelAllWatches(CHAR* Options);
extern INT32 CmdDelAllAliases(CHAR* Options);
extern void DisplayWatchValues();
extern INT32 CmdSetInput(CHAR* Options);
extern INT32 CmdSetOutput(CHAR* Options);
extern INT32 CmdSetSignal(CHAR* Options);
extern INT32 CmdSetTrace(CHAR* Options);
extern INT32 CmdSetTool(CHAR* Options);

extern INT32 ApplyResetFromType(INT32 ResetType);


/********************************************************************/
/*       D S P   T H R E A D   C L A S S							*/
/********************************************************************/
class DSP_thread : public wxThread
{
public:
	DSP_thread(wxThreadKind kind = wxTHREAD_DETACHED) : wxThread(kind) {};
	void SetDSPInstancePointer(AL3101 *pInst) {pDspInstance = pInst;};
        
	virtual ExitCode Entry();

private:
	AL3101 *pDspInstance;
};

/************************************************************/
/* NAME       :	SimulateOneSample							*/
/* DESCRIPTION:	helper function simulating one sample		*/
/*				from reading the inputs, simulating the     */
/*				instruction to saving the outputs			*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
inline INT32 SimulateOneSample(AL3101* pDspInstance)
{
	// Check if this is the first cycle of 1024 and load data
	if (pDspInstance->State.PC == 0)
	{
		// load data from file
		if (AL3101_LoadDataFromInputs(pDspInstance) != 0) return -1;
		// load data from tool
		if (AL3101_LoadDataFromTools(pDspInstance) != 0) return -1;
	}

	// Load data from signals if at least one is defined
	if (pDspInstance->NumberOfTraceInstances != 0)
		if (AL3101_LoadDataFromSignals(pDspInstance) != 0) return -1;

	// simulate a step
	MnemonicTable[pDspInstance->CurrentProgram[pDspInstance->State.PC].s_index].f_simul(&pDspInstance->State, &(pDspInstance->CurrentProgram[pDspInstance->State.PC]));
	pDspInstance->State.PC++;

	// Save data to traces if at least one is defined
	if (pDspInstance->NumberOfTraceInstances != 0)
		if (AL3101_SaveDataToTraces(pDspInstance) != 0) return -1;

	// Check if this is the last cycle of 1024 and save data
	if (pDspInstance->State.PC >= pDspInstance->State.NumberOfLines)
	{
		// write data to files
		if (AL3101_SaveDataToOutputs(pDspInstance) != 0) return -1;
		// write data to tools
		if (AL3101_SaveDataToTools(pDspInstance) != 0) return -1;
		// reset program counter
		pDspInstance->State.PC = 0;
	}

	return 0;
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
wxThread::ExitCode DSP_thread::Entry()
{
//	bool BoolResult;
	INT32 ErrorCode;
	INT32 BreakSimulation;
	INT32 CurrentNumberOfSteps;
	INT32 BreakpointReached = 0;
//	INT32 WaitTime = 10;

	DBG_PRINT("In AL3101_Process\n");

	while(true)
	{
		ErrorCode = LaunchSimulationMutex.Lock();
		if (ErrorCode != wxMUTEX_NO_ERROR) BUG_REPORT("Cannot lock mutex. Please contact Axoris.\n");
		ErrorCode = LaunchSimulation->WaitTimeout(500/*WaitTime*/);
		if (ErrorCode == wxCOND_NO_ERROR)
		{
			SimulationRunning = TRUE;
			//reset local variables
			ErrorCode = 0;
			BreakSimulation = 0;
			CurrentNumberOfSteps = 0;
			// Show user message if this is a go (and thus not a step) command.
			if (NumberOfSteps == 0)
			{
				OUTPUT_1("\rSimulation running.\n%s",PROMPT_1KSM);
				FLUSH_OUTPUT;
			}
			// Begin the "real" stuff
			while(ContinueSimulationFlag == TRUE)
			{
				// Check for number of steps
				if (NumberOfSteps != 0)									// then perform step number checking
				{
					if (CurrentNumberOfSteps == NumberOfSteps) break;	// if steps are done then break the loop
					// Update of the state copy
					memcpy(&pDspInstance->Watch.StateCopy, &pDspInstance->State, sizeof(AL3101_struct));
				}
				
				if (SimulateOneSample(pDspInstance) != 0) break;

				// Display the watches
				if (NumberOfSteps != 0) DisplayWatchValues();

				// A step has been done, increment performed step counter
				CurrentNumberOfSteps++;

				if ((NumberOfSteps != 0) && (pDspInstance->Config.ShowDisasmWhenStepping != 0)) AL3101_DisplayAssemblyLine(pDspInstance);

				// Check for breakpoints
				BreakpointReached = AL3101_CheckBreakpoints(pDspInstance);
				if (BreakpointReached != 0) break;
			}

			if (ContinueSimulationFlag == TRUE)
			{
				if (BreakpointReached == 0)		// no breakpoint reached
				{
					if (NumberOfSteps == 0)
					{
						OUTPUT_1("\nSimulation complete.\n%s",PROMPT_1KSM);
						FLUSH_OUTPUT;
						// Trying now to reset if needed
						ApplyResetFromType(pDspInstance->Config.DefaultResetType);
					}
					else if (NumberOfSteps == 1)
					{
						OUTPUT_2("\n%i step done.\n%s", NumberOfSteps, PROMPT_1KSM);
						FLUSH_OUTPUT;
					}
					else
					{
						OUTPUT_2("\n%i steps done.\n%s", NumberOfSteps, PROMPT_1KSM);
						FLUSH_OUTPUT;
					}
				}
				else
				{
					if (NumberOfSteps == 1)
					{
						OUTPUT_3("\n%i step done out of %i.\n%s", CurrentNumberOfSteps, NumberOfSteps, PROMPT_1KSM);
						FLUSH_OUTPUT;
					}
					else if (NumberOfSteps != 0)
					{
						OUTPUT_3("\n%i steps done out of %i.\n%s", CurrentNumberOfSteps, NumberOfSteps, PROMPT_1KSM);
						FLUSH_OUTPUT;
					}
				}
			}
			else
			{
				OUTPUT_1("\rUser cancelled simulation.\n%s", PROMPT_1KSM);
				FLUSH_OUTPUT;
			}

			SimulationRunning = FALSE;

			// Signal that the simulation is finished
			//   This is used when making blocking start or step
			SimulationFinished->Signal();
		} 
/*		else
		{
			if (!ThreadStarted)
			{
				ThreadStarted = 1;
				LaunchSimulationResponse->Signal();
			}
			//DBG_PRINT("AL3101_Process: Timeout or error\n"); 
		}*/
#ifndef WIN32	// this piece of code is required for a bug in Linux implementation which Gregor does not want to report.
		ErrorCode = LaunchSimulationMutex.Unlock();
#endif
//		WaitTime = 500;
		ThreadStarted = TRUE;
		if (TestDestroy() == TRUE){
		  //DBG_PRINT("Out AL3101_Process\n");
		  break;
		  //return 0;
		}
	}

	DBG_PRINT("Out AL3101_Process\n");
	return (ExitCode) 0;
}

#ifdef _STREAMING_ENABLED_
int SoundCallBack(void *inputBuffer, void *outputBuffer,
                       unsigned long bufferFrames,
                       PaTimestamp outTime, void *userData )
{
	AL3101* pDspInstance = (AL3101*)userData;
    short *in = (short*)inputBuffer;
    short *out = (short*)outputBuffer;
    //int framesToCalc = bufferFrames;
    INT32 index, aaa;
    int finished = 0;
	INT32 tempint;
//	int ReturnValue;
	INT32 NumberOfChannels = pDspInstance->StreamingParams.NumberOfChannels;

	INT32 CurrentNumberOfSteps = 0;
	INT32 BreakpointReached = 0;

	if (inputBuffer == NULL) return 0;
	if (outputBuffer == NULL) return 0;

	// Make the simulation sample per sample here
	for (index = 0; index < (int)bufferFrames; index++)
	{
		// copy first the input samples
		for (aaa = 0; aaa < NumberOfChannels; aaa++)
		{
			tempint = in[NumberOfChannels * index + aaa];		// first copy the 16 bits value in a 32 bits one
			tempint = tempint << 8;								// then prepare the data to fit a S3.24 format
			*(pDspInstance->StreamingParams.Input[aaa]) = tempint;	// copy left/mono channel to memory
		}
//		tempint = in[2 * index + 1];					// first copy the 16 bits value in a 32 bits one
//		tempint = tempint << 8;							// then prepare the data to fit a S3.24 format
//		pDspInstance->State.In[1] = tempint;			// copy left/mono channel to memory

		// Execute one instruction
		while(1)
		{
			if (SimulateOneSample(pDspInstance) != 0) break;
			// if we've made 1024 cycles, we should better store the outputs and load new inputs
			if (pDspInstance->State.PC == 0) break;
		}

		// copy then the output samples
		for (aaa = 0; aaa < NumberOfChannels; aaa++)
		{
			tempint = *(pDspInstance->StreamingParams.Output[aaa]);
			if (tempint > (INT32)0x007FFFFF) tempint = 0x007FFFFF;
			if (tempint < (INT32)0xFF800000) tempint = 0xFF800000;
			tempint = tempint >> 8;
			out[NumberOfChannels * index + aaa] = (INT16)tempint;	// copy left/mono channel to buffer
		}
//		tempint = pDspInstance->State.Out[1];
//		if (tempint > (INT32)0x007FFFFF) tempint = 0x007FFFFF;
//		if (tempint < (INT32)0xFF800000) tempint = 0xFF800000;
//		tempint = tempint >> 8;
//		out[2 * index + 1] = (INT16)tempint;			// copy left/mono channel to buffer
	}

//    return finished;
    return 0;
}
#endif

// Declare pointers for the two simulation mechanisms
DSP_thread	*pDspThread;
#ifdef _STREAMING_ENABLED_
PaStream *stream;
#endif

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 AL3101_Init(AL3101 *pDspInstance)
{
	INT32					LineCounter;
	UINT32					ReturnCode;

	/* Question: What is the influence of the stack size      */
	/* on the performance and speed of the created process ?? */
	
	DBG_PRINT(" AL3101_Init\n");

	LaunchSimulation = new wxCondition(LaunchSimulationMutex);
//	LaunchSimulationResponse = new wxCondition(LaunchSimulationResponseMutex);
	SimulationFinished = new wxCondition(SimulationFinishedMutex);

	if (!LaunchSimulation->IsOk())
	{
		ERROR("Failed to create launching synchronisation process.\n");
		return 1;
	}
/*	if (!LaunchSimulationResponse->IsOk())
	{
		ERROR("Failed to create slow-thread synchronisation process.\n");
		return 1;
	}*/
	if (!SimulationFinished->IsOk())
	{
		ERROR("Failed to create blocking synchronisation process.\n");
		return 1;
	}
	
	pDspThread=new DSP_thread;
	ReturnCode = pDspThread->Create();
	if (ReturnCode != wxTHREAD_NO_ERROR)
	{
		if (ReturnCode == wxTHREAD_NO_RESOURCE) ERROR("Unable to create DSP thread due to insufficient resources.\n");
		if (ReturnCode == wxTHREAD_RUNNING) ERROR("Unable to create DSP thread because it is already running.\n");
		return 1;
	}
	pDspThread->SetDSPInstancePointer(pDspInstance);
	//ERROR_1("Thread ID = %ld\n", pDspThread->GetId());	// Uncomment this line to get the DSP thread ID.
	ReturnCode = pDspThread->Run();
    if (ReturnCode != wxTHREAD_NO_ERROR)
    {
        ERROR("Can't start thread!");
    }
	//DspThread.Resume();

    // wait that thread is started
/*    LaunchSimulationResponseMutex.Lock();
    if (LaunchSimulationResponse->WaitTimeout(2500)!=wxCOND_NO_ERROR)
	{
		OUTPUT("Thread seems to be hard to wake up ...");
		LaunchSimulationResponseMutex.Unlock();
    }*/
	INT32 ThreadStartedIndex = 0;
	while(ThreadStarted != TRUE)
	{
		wxUsleep(10);
		ThreadStartedIndex++;
		if (ThreadStartedIndex == 250)
		{
			ERROR("Thread does not want to start.\n");
			return 1;
		}
	}

	// Initialize IO related variables
	pDspInstance->SamplingFrequency = -1;
	pDspInstance->NumberOfInputInstances = 0;
	pDspInstance->NumberOfOutputInstances = 0;
	pDspInstance->NumberOfToolInstances = 0;
	// Pre-initialize Config related variables
	//  Should be overwritten later on from registry or stuff of that kind.
	pDspInstance->Config.TrueCycleSimulation = 0;				// do not execute unneeded cycles
	pDspInstance->Config.DefaultResetType = 0;					// reset the IOs when simulation is over
	pDspInstance->Config.ShowDisasmWhenStepping = 1;			// dasm enabled by default
	pDspInstance->Config.DefaultDisplayTypeInShow = AL3101_DISPLAY_FLOAT;	// floating-point display by default in show command

	pDspInstance->Config.MaxNumberOfInputs = 32;
	pDspInstance->Config.MaxNumberOfOutputs = 32;
	pDspInstance->Config.MaxNumberOfSignals = 32;
	pDspInstance->Config.MaxNumberOfTraces = 32;
	pDspInstance->Config.MaxNumberOfTools = 32;

	// "Internal" stream parameters
	pDspInstance->StreamingParams.BlockSize = 1024;				// Let's take reasonable buffer size
	pDspInstance->StreamingParams.NumberOfBlocks = 0;			// Let PA determine the best number of blocks
#ifdef _STREAMING_ENABLED_
	pDspInstance->StreamingParams.InputDeviceID = paNoDevice;	// If no device is selected,
	pDspInstance->StreamingParams.OutputDeviceID = paNoDevice;	//  we'll get default device at start time
#endif

	// "External" stream parameters
	pDspInstance->StreamingParams.NumberOfChannels = 2;			// By default, stereo is used
	pDspInstance->StreamingParams.Input[0] = &(pDspInstance->State.In[0]);
	pDspInstance->StreamingParams.Input[1] = &(pDspInstance->State.In[1]);
	pDspInstance->StreamingParams.Input[2] = &(pDspInstance->StreamingParams.DummyValue);
	pDspInstance->StreamingParams.Input[3] = &(pDspInstance->StreamingParams.DummyValue);
	pDspInstance->StreamingParams.Input[4] = &(pDspInstance->StreamingParams.DummyValue);
	pDspInstance->StreamingParams.Input[5] = &(pDspInstance->StreamingParams.DummyValue);
	pDspInstance->StreamingParams.Input[6] = &(pDspInstance->StreamingParams.DummyValue);
	pDspInstance->StreamingParams.Input[7] = &(pDspInstance->StreamingParams.DummyValue);
	pDspInstance->StreamingParams.Output[0] = &(pDspInstance->State.Out[0]);
	pDspInstance->StreamingParams.Output[1] = &(pDspInstance->State.Out[1]);
	pDspInstance->StreamingParams.Output[2] = &(pDspInstance->StreamingParams.DummyValue);
	pDspInstance->StreamingParams.Output[3] = &(pDspInstance->StreamingParams.DummyValue);
	pDspInstance->StreamingParams.Output[4] = &(pDspInstance->StreamingParams.DummyValue);
	pDspInstance->StreamingParams.Output[5] = &(pDspInstance->StreamingParams.DummyValue);
	pDspInstance->StreamingParams.Output[6] = &(pDspInstance->StreamingParams.DummyValue);
	pDspInstance->StreamingParams.Output[7] = &(pDspInstance->StreamingParams.DummyValue);

	// Reset the breakpoints 
	for (INT32 index = 0; index <1024; index++)
		pDspInstance->Break.InstrBreakpoint[index] = 0;


	for (LineCounter=0; LineCounter<1080; LineCounter++) pDspInstance->State.Data[LineCounter] = 0;
	pDspInstance->State.A = pDspInstance->State.B = pDspInstance->State.U = 0;

	DBG_PRINT("DSP thread up and running\n");

	DBG_PRINT("Out AL3101_Init\n");
	return (0);
}


/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 AL3101_Clean(AL3101 *pDspInstance)
{
	DBG_PRINT("In AL3101_Clean\n");
	
	// Clean the IOs
	INT32 index, ErrorCode;
	for (index = 0; index < pDspInstance->NumberOfInputInstances; index++) ErrorCode = (pDspInstance->IO_Inputs[index].f_Clean)(pDspInstance->IO_Inputs[index].pInstance);
	pDspInstance->NumberOfInputInstances = 0;
	for (index = 0; index < pDspInstance->NumberOfOutputInstances; index++) ErrorCode = (pDspInstance->IO_Outputs[index].f_Clean)(pDspInstance->IO_Outputs[index].pInstance);
	pDspInstance->NumberOfOutputInstances = 0;
	for (index = 0; index < pDspInstance->NumberOfSignalInstances; index++) ErrorCode = (pDspInstance->Signals[index].f_Clean)(pDspInstance->Signals[index].pInstance);
	pDspInstance->NumberOfSignalInstances = 0;
	for (index = 0; index < pDspInstance->NumberOfTraceInstances; index++) ErrorCode = (pDspInstance->Traces[index].f_Clean)(pDspInstance->Traces[index].pInstance);
	pDspInstance->NumberOfTraceInstances = 0;
	for (index = 0; index < pDspInstance->NumberOfToolInstances; index++) ErrorCode = (pDspInstance->Tools[index].f_Clean)(pDspInstance->Tools[index].pInstance);
	pDspInstance->NumberOfToolInstances = 0;

	// Init the program counter
    pDspInstance->State.PC = 0;
	
	// Clear all breakpoints
	CmdDelAllBreakPoints(NULL);

	// Clear all watches
	CmdDelAllWatches(NULL);

	// Clear all aliases
	CmdDelAllAliases(NULL);

	//ContinueSimulationFlag = FALSE;
	if(pDspThread->IsAlive() != TRUE)
	{
		ERROR("DSP thread already killed.\n");
	}
	else pDspThread->Delete();

	delete LaunchSimulation;
	delete SimulationFinished;

	DBG_PRINT("Out AL3101_Clean\n");
	return(0);
}


/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
void AL3101_Reset_State(AL3101 *pDspInstance)
{
	DBG_PRINT("In AL3101_Reset_State\n");

	pDspInstance->State.A  = 0;
	pDspInstance->State.B  = 0;
	pDspInstance->State.U  = 0;
	pDspInstance->State.n  = 0;
	pDspInstance->State.z  = 0;
	pDspInstance->State.v  = 0;
	pDspInstance->State.PC = 0;
		
	DBG_PRINT("Out AL3101_Reset_State\n");
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 AL3101_Reset_IO(AL3101 *pDspInstance, INT32 ReOpen)
{
	INT32 index, ErrorCode;
	INT32 NumberOfInputInstances, NumberOfOutputInstances;
	INT32 NumberOfSignalInstances, NumberOfTraceInstances;
	INT32 NumberOfToolInstances;
	CHAR CmdLine[CMD_MAX_LEN];

	DBG_PRINT("In AL3101_Reset_IO\n");

	// Keep first a trace of the number of IOs, tools, ... already opened
	NumberOfInputInstances = pDspInstance->NumberOfInputInstances;
	NumberOfOutputInstances = pDspInstance->NumberOfOutputInstances;
	NumberOfSignalInstances = pDspInstance->NumberOfSignalInstances;
	NumberOfTraceInstances = pDspInstance->NumberOfTraceInstances;
	NumberOfToolInstances = pDspInstance->NumberOfToolInstances;

	// Start first with cleaning the IOs
	for (index = 0; index < NumberOfInputInstances; index++) ErrorCode = (pDspInstance->IO_Inputs[index].f_Clean)(pDspInstance->IO_Inputs[index].pInstance);
	for (index = 0; index < NumberOfOutputInstances; index++) ErrorCode = (pDspInstance->IO_Outputs[index].f_Clean)(pDspInstance->IO_Outputs[index].pInstance);
	for (index = 0; index < NumberOfSignalInstances; index++) ErrorCode = (pDspInstance->Signals[index].f_Clean)(pDspInstance->Signals[index].pInstance);
	for (index = 0; index < NumberOfTraceInstances; index++) ErrorCode = (pDspInstance->Traces[index].f_Clean)(pDspInstance->Traces[index].pInstance);
	for (index = 0; index < NumberOfToolInstances; index++) ErrorCode = (pDspInstance->Tools[index].f_Clean)(pDspInstance->Tools[index].pInstance);

	// The IOs are gone now
	pDspInstance->NumberOfInputInstances = 0;
	pDspInstance->NumberOfOutputInstances = 0;
	pDspInstance->NumberOfSignalInstances = 0;
	pDspInstance->NumberOfTraceInstances = 0;
	pDspInstance->NumberOfToolInstances = 0;
	pDspInstance->SamplingFrequency = -1;

	// Let's see if they should be reinitialized
	if (ReOpen != 0)
	{
		// Reinitialize the Input instances
		for (index = 0; index < NumberOfInputInstances; index++)
		{
			strcpy(CmdLine, pDspInstance->IO_Inputs[index].CmdLine);
			CmdSetInput(CmdLine);
		}
		// Reinitialize the Output instances
		for (index = 0; index < NumberOfOutputInstances; index++)
		{
			strcpy(CmdLine, pDspInstance->IO_Outputs[index].CmdLine);
			CmdSetOutput(CmdLine);
		}
		// Reinitialize the Signal instances
		for (index = 0; index < NumberOfSignalInstances; index++)
		{
			strcpy(CmdLine, pDspInstance->Signals[index].CmdLine);
			CmdSetSignal(CmdLine);
		}
		// Reinitialize the Traces instances
		for (index = 0; index < NumberOfTraceInstances; index++)
		{
			strcpy(CmdLine, pDspInstance->Traces[index].CmdLine);
			CmdSetTrace(CmdLine);
		}
		// Reinitialize the Tools instances
		for (index = 0; index < NumberOfToolInstances; index++)
		{
			strcpy(CmdLine, pDspInstance->Tools[index].CmdLine);
			CmdSetTool(CmdLine);
		}
	}
		
	DBG_PRINT("Out AL3101_Reset_IO\n");

	return 0;
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
void AL3101_Reset_Prog(AL3101 *pDspInstance)
{
	INT32 index;

	DBG_PRINT("In AL3101_Reset_Prog\n");

	for (index = 0; index < 1024; index++)
	{
		// Setting of these lines is absolutely necessary
		pDspInstance->CurrentProgram[index].op = 0;			// 1st opcode is used as default one
		pDspInstance->CurrentProgram[index].ln = -1;		// line number in the assembler file
		pDspInstance->CurrentProgram[index].s_index = 0;	// index in the AL3101Table (for simulation call)
		pDspInstance->CurrentProgram[index].lObj = 0;		// Object code (binary): bits  0 to 31 
		pDspInstance->CurrentProgram[index].wObj = 0;		//                       bits 32 to 40
		strcpy(pDspInstance->CurrentProgram[index].CmdStr, "CM 0.0 0x00  ; Reset by the simulator."); // string with the assembler code.
		// These lines are added just to clear everything when needed
		pDspInstance->CurrentProgram[index].m = 0;			// memory adress
		pDspInstance->CurrentProgram[index].c = 0;			// constant (aligned to S3.24 by default)
		pDspInstance->CurrentProgram[index].d = 0;			// constant (aligned to S3.24 by default)
	}
	
	// Setting the NumberOfLines to the correct value depending on the "truecyle" user parameter
	pDspInstance->State.NumberOfLines = 0;

	
	DBG_PRINT("Out AL3101_Reset_Prog\n");
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
void AL3101_Reset_Data(AL3101 *pDspInstance)
{
	INT32 index;

	DBG_PRINT("In AL3101_Reset_Data\n");
	
	for (index = 0; index < 1080; index++)
		pDspInstance->State.Data[index] = 0;
		 
	DBG_PRINT("Out AL3101_Reset_Data\n");
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
void AL3101_Reset_Breakpoints(AL3101 *pDspInstance)
{
	DBG_PRINT("In AL3101_Reset_Breakpoints\n");

	CmdDelAllBreakPoints("");
		
	DBG_PRINT("Out AL3101_Reset_Breakpoints\n");
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
void AL3101_Reset_Watches(AL3101 *pDspInstance)
{
	DBG_PRINT("In AL3101_Reset_Watches\n");
	
	CmdDelAllWatches("");
		
	DBG_PRINT("Out AL3101_Reset_Watches\n");
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
/*INT32 AL3101_Reset(AL3101 *pDspInstance)
{
	INT32 LineCounter;
	
	DBG_PRINT("In AL3101_Reset\n");
	
	for (LineCounter=0;LineCounter<1080;LineCounter++) pDspInstance->State.Data[LineCounter]=0;
	pDspInstance->State.A = pDspInstance->State.B = pDspInstance->State.U = 0;
		
	DBG_PRINT("Out AL3101_Reset\n");
	return(0);
}*/


/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 AL3101_Start(AL3101* pDspInstance, INT32 nSteps, INT32 BlockingMode)
{
	DBG_PRINT("In AL3101_Start\n");

	if ((ContinueSimulationFlag == TRUE) && (SimulationRunning == TRUE))
	{	// We should not do anything, a simulation is already going on.
		ERROR("A simulation is already running, please stop before starting again.\n");
		DBG_PRINT("Out AL3101_Start\n");
		return(-1);
	}
	// At this stage, we know we can launch a simulation
	ContinueSimulationFlag = TRUE;
	NumberOfSteps = nSteps;

	if (BlockingMode == 0)
	{	// We don't have to wait, just launch simulation
		LaunchSimulation->Signal();
	}
	else
	{	// We will wait for simulation to be finished if we are in blocking mode
		SimulationFinishedMutex.Lock();
		LaunchSimulation->Signal();
		while(true)
		{
			INT32 ErrorCode = SimulationFinished->WaitTimeout(1000);
			if (ErrorCode == wxCOND_NO_ERROR) break;
		}
	}
	
	
	DBG_PRINT("Out AL3101_Start\n");
	return(0);
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 AL3101_Stop(AL3101* pDspInstance)
{
	DBG_PRINT("In AL3101_Stop\n");

	ContinueSimulationFlag = FALSE;

	DBG_PRINT("Out AL3101_Stop\n");
	return(0);
}


/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 AL3101_StartStream(AL3101* pDspInstance)
{
	DBG_PRINT("In AL3101_StartStream\n");

#ifdef _STREAMING_ENABLED_
    PaError err;
	if (IsStreaming == 1)
	{	// We should not do anything, a simulation is already going on.
		ERROR("A streaming simulation is already running, please stopstream before starting again.\n");
		DBG_PRINT("Out AL3101_StartStream\n");
		return(-1);
	}

	if ((ContinueSimulationFlag == TRUE) && (SimulationRunning == TRUE))
	{	// We should not do anything, a simulation is already going on.
		ERROR("A simulation is already running, please stop before starting again.\n");
		DBG_PRINT("Out AL3101_StartStream\n");
		return(-1);
	}
	// At this stage, we know we can launch a simulation
	ContinueSimulationFlag = TRUE;
	NumberOfSteps = 0;

	// We don't have to wait, just launch simulation
	//LaunchSimulation->Signal();

    /* initialise portaudio subsytem */
    err = Pa_Initialize();
	if (err != paNoError)
	{
		ERROR("Cannot initialize sound library.\n");
		DBG_PRINT("Out AL3101_StartStream\n");
		return -1;
	}

	//const PaDeviceInfo *padev = Pa_GetDeviceInfo(outputDevice);
	if (pDspInstance->StreamingParams.InputDeviceID == paNoDevice)
		pDspInstance->StreamingParams.InputDeviceID = Pa_GetDefaultInputDeviceID();
	if (pDspInstance->StreamingParams.OutputDeviceID == paNoDevice)
		pDspInstance->StreamingParams.OutputDeviceID = Pa_GetDefaultOutputDeviceID();

	printf("BlockSize = %i\n", pDspInstance->StreamingParams.BlockSize);
	printf("nChane = %i\n", pDspInstance->StreamingParams.NumberOfChannels);
	err = Pa_OpenStream(
			&stream,
            pDspInstance->StreamingParams.InputDeviceID,
            pDspInstance->StreamingParams.NumberOfChannels,		/* ? input */
            paInt16,											/* 16 bits integer input */
            NULL,
            pDspInstance->StreamingParams.OutputDeviceID,
            pDspInstance->StreamingParams.NumberOfChannels,		/* ? output */
            paInt16,											/* 16 bits integer output */
            NULL,
            (double)pDspInstance->SamplingFrequency,
            pDspInstance->StreamingParams.BlockSize,			/* buffer size */
            pDspInstance->StreamingParams.NumberOfBlocks,		/* number of buffers */
            paClipOff,      /* we won't output out of range samples so don't bother clipping them */
            SoundCallBack,
            (void*) pDspInstance);

    if( err == paNoError )
    {
        err = Pa_StartStream( stream );
    }
    else
    {
        if( err == paHostError )
            ERROR_1("%s\n", Pa_GetErrorText(Pa_GetHostError()) );
        else
            ERROR_1("Error number: %d\n", err );

		Pa_Terminate();

		DBG_PRINT("Out AL3101_StartStream\n");
		return -2;
    }

	IsStreaming = 1;
#else
	ERROR("\nStreaming is not enabled in this executable version.\nPlease use a streaming enable one.\n");
#endif

	
	DBG_PRINT("Out AL3101_StartStream\n");
	return(0);
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 AL3101_StopStream(AL3101* pDspInstance)
{
	DBG_PRINT("In AL3101_StopStream\n");

#ifdef _STREAMING_ENABLED_
    PaError err;

	if (IsStreaming == 1)
	{
		ContinueSimulationFlag = FALSE;
		err = Pa_StopStream( stream );
		while (Pa_StreamActive( stream ) != FALSE) Pa_Sleep(200);
		err = Pa_CloseStream( stream );
		Pa_Terminate();
		IsStreaming = 0;
	}
	else
	{
		ERROR("Cannot stop stream because stream is not started.\n");

	}
#else
	ERROR("\nStreaming is not enabled in this executable version.\nPlease use a streaming enable one.\n");
#endif

	DBG_PRINT("Out AL3101_StopStream\n");
	return(0);
}


/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 _PrintDSPState()
{
	CHAR State[10];

	if (ContinueSimulationFlag == TRUE) {strcpy(State, "TRUE");}
	else if(ContinueSimulationFlag == FALSE) {strcpy(State, "FALSE");}
		else strcpy(State, "ERROR");
	OUTPUT_1("ContinueSimulationFlag = <%s>\n", State);
	return 0;
}




// Helper functions for the simulation
inline INT32 AL3101_LoadDataFromInputs(AL3101 *pDspInstance)
{
	INT32 BreakSimulation = 0;
	INT32 ErrorCode = 0;
	INT32 index;
	CHAR str[256];
	
	// load data from file
	for (index = 0; index < pDspInstance->NumberOfInputInstances; index++)
	{
		ErrorCode = (pDspInstance->IO_Inputs[index].f_Action)(pDspInstance->IO_Inputs[index].pInstance);
		if (ErrorCode != FILEIO_NOERROR)
		{
			BreakSimulation = 1;
			if (ErrorCode != FILEIO_EOF_REACHED)
			{
				ERROR("Problem in reading input file\n");
				FileIO_GetLastErrorString(str, ErrorCode, 256);
				ERROR_1("%s\n", str);
			}
		}
	}

	return BreakSimulation;
}

inline INT32 AL3101_LoadDataFromSignals(AL3101 *pDspInstance)
{
	INT32 BreakSimulation = 0;
	INT32 ErrorCode = 0;
	INT32 index;
	CHAR str[256];
	
	// load data from file
	for (index = 0; index < pDspInstance->NumberOfSignalInstances; index++)
	{
		ErrorCode = (pDspInstance->Signals[index].f_Action)(pDspInstance->Signals[index].pInstance);
		if (ErrorCode != FILEIO_NOERROR)
		{
			BreakSimulation = 1;
			if (ErrorCode != FILEIO_EOF_REACHED)
			{
				ERROR("Problem in reading signal file\n");
				FileIO_GetLastErrorString(str, ErrorCode, 256);
				ERROR_1("%s\n", str);
			}
		}
	}

	return BreakSimulation;
}

inline INT32 AL3101_LoadDataFromTools(AL3101 *pDspInstance)
{
	INT32 BreakSimulation = 0;
	INT32 ErrorCode = 0;
	INT32 index;

	// load data from tool
	for (index = 0; index < pDspInstance->NumberOfToolInstances; index++)
	{
		if (pDspInstance->Tools[index].f_Read != NULL) ErrorCode = (pDspInstance->Tools[index].f_Read)(pDspInstance->Tools[index].pInstance);
		if (ErrorCode != 0)
		{
			BreakSimulation = 1;
			/*if (ErrorCode != FILEIO_EOF_REACHED)
			{
				ERROR(stderr, "Problem in reading input file\n");
				FileIO_GetLastErrorString(str, ErrorCode, 256);
				ERROR_1("%s\n", str);
			}*/
		}
	}

	return BreakSimulation;
}

inline INT32 AL3101_SaveDataToOutputs(AL3101 *pDspInstance)
{
	INT32 BreakSimulation = 0;
	INT32 ErrorCode = 0;
	INT32 index;
	CHAR str[256];

	// write data to files
	for (index = 0; index < pDspInstance->NumberOfOutputInstances; index++)
	{
		ErrorCode = (pDspInstance->IO_Outputs[index].f_Action)(pDspInstance->IO_Outputs[index].pInstance);
		if (ErrorCode != FILEIO_NOERROR)
		{
			BreakSimulation = 1;
			if (ErrorCode != FILEIO_EOF_REACHED)
			{
				ERROR("Problem in writing output file\n");
				FileIO_GetLastErrorString(str, ErrorCode, 256);
				ERROR_1("%s\n", str);
			}
		}
	}

	return BreakSimulation;
}

inline INT32 AL3101_SaveDataToTraces(AL3101 *pDspInstance)
{
	INT32 BreakSimulation = 0;
	INT32 ErrorCode = 0;
	INT32 index;
	CHAR str[256];

	// write data to files
	for (index = 0; index < pDspInstance->NumberOfTraceInstances; index++)
	{
		ErrorCode = (pDspInstance->Traces[index].f_Action)(pDspInstance->Traces[index].pInstance);
		if (ErrorCode != FILEIO_NOERROR)
		{
			BreakSimulation = 1;
			if (ErrorCode != FILEIO_EOF_REACHED)
			{
				ERROR("Problem in writing trace file\n");
				FileIO_GetLastErrorString(str, ErrorCode, 256);
				ERROR_1("%s\n", str);
			}
		}
	}

	return BreakSimulation;
}

inline INT32 AL3101_SaveDataToTools(AL3101 *pDspInstance)
{
	INT32 BreakSimulation = 0;
	INT32 ErrorCode = 0;
	INT32 index;

	// write data to tools
	for (index = 0; index < pDspInstance->NumberOfToolInstances; index++)
	{
		if (pDspInstance->Tools[index].f_Write != NULL) ErrorCode = (pDspInstance->Tools[index].f_Write)(pDspInstance->Tools[index].pInstance);
		if (ErrorCode != 0)
		{
			BreakSimulation = 1;
			/*if (ErrorCode != FILEIO_EOF_REACHED)
			{
				ERROR("Problem in writing output file\n");
				FileIO_GetLastErrorString(str, ErrorCode, 256);
				ERROR_1("%s\n", str);
			}*/
		}
	}

	return BreakSimulation;
}

inline INT32 AL3101_CheckBreakpoints(AL3101* pDspInstance)
{
	INT32 index;
	INT32 ReturnValue = pDspInstance->Break.InstrBreakpoint[pDspInstance->State.PC];

	// Check if usual breakpoint reached
	if (pDspInstance->Break.InstrBreakpoint[pDspInstance->State.PC] == 1)
	{
		OUTPUT_1("Reached breakpoint at address %i\n", pDspInstance->State.PC);
		return ReturnValue;	
	}
	// Check if temporary breakpoint reached
	if (pDspInstance->Break.InstrBreakpoint[pDspInstance->State.PC] > 1)
	{
		// decrement the hit count
		pDspInstance->Break.InstrBreakpoint[pDspInstance->State.PC] -= 1;
		OUTPUT_2("Reached temporary breakpoint at address %i. %i breaks remaining.\n", pDspInstance->State.PC, (pDspInstance->Break.InstrBreakpoint[pDspInstance->State.PC] - 1));
		if (pDspInstance->Break.InstrBreakpoint[pDspInstance->State.PC] == 1)
		{
			pDspInstance->Break.InstrBreakpoint[pDspInstance->State.PC] = 0;
			OUTPUT("Number of hits for temporary breakpoint exhausted: clearing breakpoint.\n");
		}
		return ReturnValue;	
	}
	// Check if value breakpoint reached
	INT32 BreakCount = pDspInstance->Break.ValBreakpoints.GetCount();
	if (BreakCount != 0)
	{
		AL3101_Value_Breakpoint	*Breakpoint;
		ValBreakpoint_List::Node *Node;

		ReturnValue = 0;
		pDspInstance->Break.BreakpointProcessingInProgress.Enter();
		for (index = 0; index < BreakCount; index++)
		{
			Node = pDspInstance->Break.ValBreakpoints.Item(index);
			Breakpoint = Node->GetData();
			if (*(Breakpoint->Address) == Breakpoint->TestValue)
			{
				OUTPUT_2("Breaking because %s = %i\n", Breakpoint->DisplayString, Breakpoint->TestValue);
				ReturnValue = 1;
			}
		}
		pDspInstance->Break.BreakpointProcessingInProgress.Leave();
		return ReturnValue;	
	}
		
	return ReturnValue;	
}

inline void AL3101_DisplayAssemblyLine(AL3101* pDspInstance)
{
	INT32 CurrentAddress = pDspInstance->State.PC;

	//OUTPUT_3("Executed opcode at %i: %.2x%.8x\n", CurrentAddress, pDspInstance->CurrentProgram[CurrentAddress].wObj, pDspInstance->CurrentProgram[CurrentAddress].lObj);
	OUTPUT_2("#%.4d: %s\n", CurrentAddress, pDspInstance->CurrentProgram[CurrentAddress].CmdStr);
}

