/***********************************************************************************************
 *
 * Functions	: CFTriGenerator v1.0
 *
 * Members Functions:
 *
 *		1. Initialization
 *			void SinGenerator_Init(SinGeneratorStruct *pInst, float SamplingFrequency, float Frequency, float Amplitude; float InitialPhase)			
 *
 *		2. Set Functions
 *			void SinGenerator_SetSamplingFrequency(SinGeneratorStruct *pInst, float SamplingFrequency)
 *			void SinGenerator_SetFrequency(SinGeneratorStruct *pInst, float Frequency)
 *			void SinGenerator_SetAmplitude(SinGeneratorStruct *pInst, float Amplitude)
 *			void SinGenerator_SetInitialPhase(SinGeneratorStruct *pInst, float InitialPhase)
 *		
 *		3. Process Function
 *			void SinGenerator_Process(SinGeneratorStruct *pInst, float *pOut) 
 *          void SinGenerator_ProcessBlock(SinGeneratorStruct *pInst, float *pOut, INT32 BufLen)
 *
 ************************************************************************************************/
#include "CFTriGenerator.h"
#include "math.h"				// used by sin/cos 1st calc.


// Initialization
void CFTriGenerator::Init(float ParSamplingFrequency, float ParFrequency, float ParAmplitude, float ParInitialPhase, int ParMode)
{
	SamplingFrequency = ParSamplingFrequency;
	Frequency = ParFrequency;
	Amplitude = ParAmplitude;
	InitialPhase = ParInitialPhase;
	Mode = ParMode;

	ValueIncrement = 4.0f * Frequency / SamplingFrequency;
	CurrentValue = 0.0f;
}																 


// Set Functions
void CFTriGenerator::SetSamplingFrequency(float ParSamplingFrequency)
{
	SamplingFrequency = ParSamplingFrequency;

	ValueIncrement = 4.0f * Frequency / SamplingFrequency;
}

void CFTriGenerator::SetFrequency(float ParFrequency)
{
	Frequency = ParFrequency;

	ValueIncrement = 4.0f * Frequency / SamplingFrequency;
}

void CFTriGenerator::SetAmplitude(float ParAmplitude)
{
	Amplitude = ParAmplitude;
}

void CFTriGenerator::SetInitialPhase(float ParInitialPhase, INT32 ResetCurrentPhase)
{
	InitialPhase = ParInitialPhase;

	if (ResetCurrentPhase != 0) Init(SamplingFrequency, Frequency, Amplitude, InitialPhase, Mode);
}

void CFTriGenerator::Reset()
{
	Init(SamplingFrequency, Frequency, Amplitude, InitialPhase, Mode);
}		

// Process Function
void CFTriGenerator::Process(float *pOut)
{
	// writing in buffer
	*pOut = CurrentValue * Amplitude;

	// generation of the triangle
	CurrentValue += ValueIncrement;

	// Check overflow
	if (Mode == 0)		// We should generate a triangle
	{
		if (CurrentValue > 1.0f)
		{
			CurrentValue = 2.0f - CurrentValue;
			ValueIncrement = -ValueIncrement;
		}
		if (CurrentValue < -1.0f)
		{
			CurrentValue = -2.0f - CurrentValue;
			ValueIncrement = -ValueIncrement;
		}
	}
	else				// We should generate a sawtooth
	{
		if (CurrentValue > 1.0f) CurrentValue = -1.0f;
	}
}

void CFTriGenerator::ProcessBlock(float *pOut, INT32 BufLen)
{
	INT32 index;

	for (index = 0; index < BufLen; index++)
	{
		// writing in buffer
		*pOut = CurrentValue * Amplitude;
		pOut++;

		// generation of the triangle
		CurrentValue += ValueIncrement;
		
		// Check overflow
		if (Mode == 0)		// We should generate a triangle
		{
			if (CurrentValue > 1.0f)
			{
				CurrentValue = 2.0f - CurrentValue;
				ValueIncrement = -ValueIncrement;
			}
			if (CurrentValue < -1.0f)
			{
				CurrentValue = -2.0f + CurrentValue;
				ValueIncrement = -ValueIncrement;
			}
		}
		else				// We should generate a sawtooth
		{
			if (CurrentValue > 1.0f) CurrentValue = -1.0f;
		}
	}
}